//=========================================================================
//
//	FILE:				CP_DemoPathsView.cpp
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			CP_View to display paths using CP_Graphics.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 7 $
//						Latest Checkout:  $Date: 2005-08-30 20:31:18 -0700 (Tue, 30 Aug 2005) $
//
//=========================================================================

	// Precompile header support
#include "CP_Precomp.h"

	// CP_Graphics
#include "CP_DemoPathsView.h"

	// CPLAT_Framework
#include "CP_Debugging.h"
#include "CP_Errors.h"
#include "CP_Exceptions.h"
#include "CP_Gradient.h"
#include "CP_Graphics.h"
#include "CP_Types.h"

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Construction / Destruction ===
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	CP_DemoPathsView
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Constructor.
*
*	@param		inSuperview			The CP_View that contains this view.
*	@param		inUpperLeft			Upperleft coordinate of view.
*	@param		inContentSize		The size of the view.
*	
*	@exception	none
*/
CP_DemoPathsView::CP_DemoPathsView( CPLAT::CP_View *inSuperview,
	const CPLAT::CP_Point& inUpperLeft, const CPLAT::CP_Size& inContentSize )
		: CPLAT::CP_View( inSuperview, inUpperLeft, inContentSize, true )
{
}

//=========================================================================
//	FUNCTION:	~CP_DemoPathsView
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Destructor.
*
*	@exception	none
*/
CP_DemoPathsView::~CP_DemoPathsView() throw()
{
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Drawing ===
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	OnDraw
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Perform the actual drawing of the view.
*
*	@param		inCanvas			CP_Canvas object to draw into.
*	@param		inUpdateArea		CP_Region containing the area to draw.
*	
*	@exception	none
*
*	@return		void
*/
void CP_DemoPathsView::OnDraw( CPLAT::CP_Canvas& inCanvas, CPLAT::CP_Region& /*inUpdateArea*/ )
{
	CPLAT::CP_Graphics			graphicsOne( inCanvas );
	SInt16						verPosn = 5;
	const float					kSize = (150 * 0.9);
	const float					kHalfSize = (kSize * 0.5);
	const float					kTenthSize = (kSize * 0.1);
	SInt16						x = 0;
	CPLAT::CP_Point				points[ 6 ];
	CPLAT::CP_ARGBColor			fillColor( 0.0, 1.0, 0.0, 1.0 );
	CPLAT::CP_ARGBColor			strokeColor( 1.0, 0.0, 0.0, 1.0 );
	SInt16						verOffset = 150;
	SInt16						horOffset = 150;
		
	points[0].Set( kTenthSize, 0 );
	points[1].Set( kHalfSize, kSize - kTenthSize );
	points[2].Set( kSize - kTenthSize, 0 );
	points[3].Set( 0, kHalfSize );
	points[4].Set( kSize, kHalfSize );
	points[5].Set( kTenthSize, 0 );
			
		// Fill Path
	graphicsOne.SetFillColor( fillColor );
	graphicsOne.BeginPath();
	graphicsOne.PathAddLines( points, 6 );
	graphicsOne.ClosePath();
	graphicsOne.FillPath();
		
		// Stroke path
	graphicsOne.SetStrokeColor( strokeColor );
	graphicsOne.SetLineWidth( 2 );
	graphicsOne.BeginPath();
	graphicsOne.PathAddLines( points, 6 );
	graphicsOne.ClosePath();
	graphicsOne.StrokePath();

	CPLAT::CP_Point_Array			pointArray;
	
	pointArray.push_back( CPLAT::CP_Point( 200, 10 ) );
	pointArray.push_back( CPLAT::CP_Point( 250, 50 ) );
	pointArray.push_back( CPLAT::CP_Point( 200, 60 ) );
	pointArray.push_back( CPLAT::CP_Point( 200, 10 ) );			// close shape
	
	graphicsOne.BeginPath();
	graphicsOne.PathAddPoly( pointArray );
	graphicsOne.StrokePath();

	graphicsOne.SetGradientCount( 3 );
	graphicsOne.SetGradientColor( 1, CPLAT::CP_ARGBColor( 1, 0, 0, 1 ) );
	graphicsOne.SetGradientColor( 2, CPLAT::CP_ARGBColor( 0, 1, 0, 1 ) );
	graphicsOne.SetGradientColor( 3, CPLAT::CP_ARGBColor( 0, 0, 1, 1 ) );

	CPLAT::CP_Rect		boundsOne( 0, 150, 150, 300 );
	graphicsOne.DrawGradient( CPLAT::CP_E_GradientType_TopToBottom, boundsOne );

		// Clip path
	points[0].Set( kTenthSize + horOffset, 0 + verOffset );
	points[1].Set( kHalfSize + horOffset, (kSize - kTenthSize) + verOffset );
	points[2].Set( (kSize - kTenthSize) + horOffset, 0 + verOffset );
	points[3].Set( 0 + horOffset, kHalfSize + verOffset );
	points[4].Set( kSize + horOffset, kHalfSize + verOffset );
	points[5].Set( kTenthSize + horOffset, 0 + verOffset );

	graphicsOne.BeginPath();
	graphicsOne.PathAddLines( points, 6 );
	graphicsOne.ClosePath();
	graphicsOne.ClipPath();

		// draw sunburst gradient in above clip path
	CPLAT::CP_Rect		bounds( 150, 150, 300, 300 );
	graphicsOne.DrawGradient( CPLAT::CP_E_GradientType_Sunburst, bounds );
}

#if TARGET_OS_MAC

#pragma mark -
#pragma mark === Mac specific ===
#pragma mark -

#endif	// TARGET_OS_MAC

#if TARGET_OS_WIN32

#pragma mark -
#pragma mark === Win specific ===
#pragma mark -

#endif	// TARGET_OS_WIN32

#if TARGET_OS_LINUX

#pragma mark -
#pragma mark === Linux specific ===
#pragma mark -

#endif	// TARGET_OS_LINUX