//=========================================================================
//
//	FILE:				CP_FinderLike_TV_DB_Doc.cpp
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			CP_Document class to show the CP_TV_DataBrower in a
//						simple single column tree view.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 6 $
//						Latest Checkout:  $Date: 2005-09-05 19:25:45 -0700 (Mon, 05 Sep 2005) $
//
//=========================================================================

	// Document
#include "CP_FinderLike_TV_DB_Doc.h"

	// CPLAT_Framework
#include "CP_Debugging.h"
#include "CP_Document.tmpl.h"
#include "CP_DocumentWindow.h"
#include "CP_FileIterator.h"
#include "CP_FileSystem.h"
#include "CP_New.h"
#include "CP_MenuBarManager.h"
#include "CP_OSStatus.h"
#include "CP_Scroller.h"
#include "CP_SignalSlot.h"
#include "CP_TableHeaders.h"
#include "CP_TV_DataBrowser.h"
#include "CP_TV_DataBrowserHandlers.h"
#include "CP_TV_DataBrowserRow.h"
#include "CP_TV_DataBrowser_Defines.h"
#include "CP_Types.h"
#include "CP_Utility.h"
#include "CP_Window.h"
#include "CP_WindowManager.h"

const UInt32 kNameDataSourceIDOffset = 0;
const UInt32 kModifiedDataSourceIDOffset = 10000;
const UInt32 kSizeDataSourceIDOffset = 20000;
const UInt32 kKindDataSourceIDOffset = 30000;

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Construction / Destruction ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	CP_FinderLike_TV_DataBrowser_Doc
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Constructor.
*
*	@param		inContainer			The scriptable object that serves as a container
*									for this document. In this case it will be the
*									application object.
*	@param		inDocType			The documents default file type.
*	
*	@exception	none
*/
CP_FinderLike_TV_DataBrowser_Doc::CP_FinderLike_TV_DataBrowser_Doc( CPLAT::CP_ScriptableObject *inContainer, CPLAT::FourCharCode inDocType )
	: inherited( inContainer, inDocType )
{
	fFileSpecsDataSource = CP_new CP_DataSource_FileSpecs;
}

//=========================================================================
//	FUNCTION:	~CP_FinderLike_TV_DataBrowser_Doc
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Destructor.
*
*	@exception	none
*/
CP_FinderLike_TV_DataBrowser_Doc::~CP_FinderLike_TV_DataBrowser_Doc() throw()
{
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Creation ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	DocumentCreator
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Called when some action has requested we create a new document object. This
*	is usually called from the CP_DocumentFactoryManager when it needs to create
*	a new document.
*
*	@param		inContainer			The scriptable object that serves as a container
*									for this document. In this case it will be the
*									application object.
*	@param		inDocumentClass		The documents default file type.
*	
*	@exception	none
*
*	@return		void
*/
CPLAT::CP_Document_Base *CP_FinderLike_TV_DataBrowser_Doc::DocumentCreator( CPLAT::CP_ScriptableObject *inContainer,
	CPLAT::FourCharCode inDocumentClass )
{
	return( CP_new CP_FinderLike_TV_DataBrowser_Doc( inContainer, inDocumentClass ) );
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === FileSystem ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	GetListingDirectory
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Called to ask the user to select the directory to list the files from.
*	
*	@exception	none
*
*	@return		void
*/
void CP_FinderLike_TV_DataBrowser_Doc::GetListingDirectory()
{
	try
	{
		CPLAT::CP_GetDirectoryDialog			*getDirectoryDialog = NULL;
		CPLAT::CP_FileSelectionCallback			*getDirectoryResultSignal = NULL;
		
		getDirectoryDialog = CP_new CPLAT::CP_GetDirectoryDialog;
		getDirectoryDialog->SetModality( CPLAT::kCP_E_WindowModalityAppModal, NULL );

		getDirectoryResultSignal = CP_new CPLAT::CP_FileSelectionCallback();
		getDirectoryResultSignal->connect( CPLAT::CP_SigSlot::slot(this, &CP_FinderLike_TV_DataBrowser_Doc::GetDirectory_Callback) );

		getDirectoryDialog->SetFileSelectionCallback( getDirectoryResultSignal );

		getDirectoryDialog->Run();
	}
	catch(...)
	{
	}
}

//=========================================================================
//	FUNCTION:	GetDirectory_Callback
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Callback from the GetDirectory dialog when the user has selected the directory
*	to list files from.
*
*	@param		inSendingDialog			Not used.
*	@param		inSelection				The selected directory.
*	@param		inOverwrite				Not used.
*	
*	@exception	none
*
*	@return		void
*/
void CP_FinderLike_TV_DataBrowser_Doc::GetDirectory_Callback( CPLAT::CP_FileSystemDialogs_Base */*inSendingDialog*/,
	const CPLAT::CP_FileSystemSpec& inSelection, bool /*inOverwrite*/ )
{
	fDirectory = inSelection;
	
	if( fFileSpecsDataSource )
	{
			// fill in the file specs dialog with all of the files in this directory
		CPLAT::CP_FileIterator				directoryIterator( inSelection, NULL );
		CPLAT::CP_FileSystemSpec_Array		fileSpecsArray = directoryIterator.GetFileSpecs();
		CPLAT::CP_FileSystemSpec_Iterator	iter = fileSpecsArray.begin();
		CPLAT::CP_FileSystemSpec			nextSpec;
		
		while( iter != fileSpecsArray.end() )
		{
			nextSpec = *iter++;
			
			fFileSpecsDataSource->AddFileSpec( nextSpec );
		}
			
		HandleCreateDocumentUI( CPLAT::CP_Document_Base::CP_E_Doc_Create_CreateUIOnNew );
	}
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Main Window ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	HandleCreateDocumentUI
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Called when the document wants to create the main user interface for the
*	document. In this demo we create a window, with a CP_TextView in it.
*
*	@param		inPostCreateFlags		Not used.
*	
*	@exception	none
*
*	@return		void
*/
void CP_FinderLike_TV_DataBrowser_Doc::HandleCreateDocumentUI( SInt32 inPostCreateFlags )
{
	try
	{
		CPLAT::CP_DocumentWindow		*window = NULL;
		CPLAT::CP_Point					windowLocation( 10, 50 );
		CPLAT::CP_Size					windowSize( 750, 200 );
		CPLAT::CP_String				windowTitle( "Finder Like CP_TV_DataBrowser" );
		CPLAT::CP_DocumentWindowType	documentWindowType;
		CPLAT::CP_Scroller				*tableScroller = NULL;
		
		window = CP_new CPLAT::CP_DocumentWindow();
		window->Create( windowLocation, windowSize, windowTitle, documentWindowType );

		CPLAT::CP_MenuBarManager::Instance()->GetCurrentMenuBar()->SetMainWindow( window );

		CPLAT::CP_Point					viewLocation( 0, 0 );
		CPLAT::CP_Size					viewSize( 750, 200 );
		CPLAT::CP_Size					scrollUnit( 20, 20 );
		CPLAT::CP_S_TV_DB_Properties	finderLikeDataBrowserProperties;
		CPLAT::CP_TV_DataBrowser		*finderLikeDataBrowser = NULL;
		CPLAT::CP_TableHeaders			*finderLikeTableHeaders = NULL;
		
		viewLocation.Set( 0, 0 );
		viewSize.Set( 750, 18 );
		finderLikeTableHeaders = CP_new CPLAT::CP_TableHeaders( window, viewLocation, viewSize );
		finderLikeTableHeaders->SetFrameAdapter( CP_new CPLAT::CP_FrameAdapter_Sticky( true, true, true, false ) );

		CPLAT::CP_S_TableHeaderItemInfo			nameItemInfo( CPLAT::CP_String("Name"),
			CPLAT::CP_CMD_NoCommand, CPLAT::CP_E_TextAlign_HorLeft, CPLAT::kCP_NoImageIndex, 300, 0, 100 );
		CPLAT::CP_S_TableHeaderItemInfo			modifiedItemInfo( CPLAT::CP_String("Modified"),
			CPLAT::CP_CMD_NoCommand, CPLAT::CP_E_TextAlign_HorLeft, CPLAT::kCP_NoImageIndex, 200, 0, 100 );
		CPLAT::CP_S_TableHeaderItemInfo			sizeItemInfo( CPLAT::CP_String("Size"),
			CPLAT::CP_CMD_NoCommand, CPLAT::CP_E_TextAlign_HorRight, CPLAT::kCP_NoImageIndex, 100, 0, 50 );
		CPLAT::CP_S_TableHeaderItemInfo			kindItemInfo( CPLAT::CP_String("Kind"),
			CPLAT::CP_CMD_NoCommand, CPLAT::CP_E_TextAlign_HorLeft, CPLAT::kCP_NoImageIndex, 150, 0, 75 );
		finderLikeTableHeaders->AddItem( nameItemInfo );
		finderLikeTableHeaders->AddItem( modifiedItemInfo );
		finderLikeTableHeaders->AddItem( sizeItemInfo );
		finderLikeTableHeaders->AddItem( kindItemInfo );

		viewLocation.Set( 0, 18 );
		viewSize.Set( 750, 200 - 18 );
		tableScroller = CP_new CPLAT::CP_Scroller( window, viewLocation, viewSize );
		tableScroller->SetFrameAdapter( CP_new CPLAT::CP_FrameAdapter_Sticky( true, true, true, true ) );

		viewLocation.Set( 0, 0 );
		viewSize.Set( 600 - 16, 200 - (18 + 16) );
		finderLikeDataBrowserProperties.fNumberColumns = 4;
		finderLikeDataBrowserProperties.fGeometryType = CPLAT::kCP_TV_GeometryMulti;
		finderLikeDataBrowser = CP_new CPLAT::CP_TV_DataBrowser( tableScroller, viewLocation, viewSize, finderLikeDataBrowserProperties );
		finderLikeDataBrowser->SetFrameAdapter( CP_new CPLAT::CP_FrameAdapter_Sticky( true, true, true, true ) );
		
		tableScroller->SetScrollingView( finderLikeDataBrowser );
		finderLikeDataBrowser->SetScrollUnit( scrollUnit );

		finderLikeDataBrowser->SetDataSource( fFileSpecsDataSource );

			// set up the tables content handlers
		CPLAT::CP_TV_DataBrowser_CH_IconAndText *iconHandler = CP_new CPLAT::CP_TV_DataBrowser_CH_IconAndText();
		CPLAT::CP_TV_DataBrowser_CH_String		*stringHandler = CP_new CPLAT::CP_TV_DataBrowser_CH_String();
		
		CPLAT::CP_FontSpec						fontSpec;
		
		#if TARGET_OS_MAC
			fontSpec.SetOSFontNumber( kThemeViewsFont );
		#endif
			
		CPLAT::CP_DataValueHolder				dataValueHolder( fontSpec );
		iconHandler->SetProperty( kCP_ContentHandler_FontSpec, dataValueHolder );
		stringHandler->SetProperty( kCP_ContentHandler_FontSpec, dataValueHolder );
		
		CPLAT::CP_S_TV_DB_ContentHandlerInfo		iconAndTextContentHandlerInfo( CPLAT::kCP_Content_IconAndText, iconHandler );
		finderLikeDataBrowser->RegisterHandler( iconAndTextContentHandlerInfo );

		CPLAT::CP_S_TV_DB_ContentHandlerInfo		stringContentHandlerInfo( CPLAT::kCP_Content_String, stringHandler );
		finderLikeDataBrowser->RegisterHandler( stringContentHandlerInfo );

		finderLikeDataBrowser->SetColumnContentType( 1, CPLAT::kCP_Content_IconAndText );
		finderLikeDataBrowser->SetColumnContentType( 2, CPLAT::kCP_Content_String );
		finderLikeDataBrowser->SetColumnContentType( 3, CPLAT::kCP_Content_String );
		finderLikeDataBrowser->SetColumnContentType( 4, CPLAT::kCP_Content_String );

		finderLikeDataBrowser->SetColumnWidth( -1, 1, 300 );
		finderLikeDataBrowser->SetColumnWidth( -1, 2, 200 );
		finderLikeDataBrowser->SetColumnWidth( -1, 3, 100 );
		finderLikeDataBrowser->SetColumnWidth( -1, 4, 150 );
		finderLikeDataBrowser->SetRowHeight( -1, 20 );
		
		CPLAT::CP_S_TV_DB_ColumnParams					columnParams;
		SInt16											horTextAlign = CPLAT::CP_E_TextAlign_HorRight;
		CPLAT::CP_DataValueHolder						textAlignDataValueHolder( horTextAlign );

			// column 1 - Name
		columnParams = finderLikeDataBrowser->GetColumnParams( 1 );
		columnParams.fColumnOffset = 6;
		finderLikeDataBrowser->SetColumnParams( 1, columnParams );

			// column 2 - Modified
		columnParams = finderLikeDataBrowser->GetColumnParams( 2 );
		columnParams.fColumnOffset = 6;
		finderLikeDataBrowser->SetColumnParams( 2, columnParams );

			// column 3 - Size
		columnParams = finderLikeDataBrowser->GetColumnParams( 3 );
		columnParams.fColumnProperties.AddValue( kCP_ContentHandler_HorAlign, textAlignDataValueHolder );
		columnParams.fColumnOffset = 6;
		finderLikeDataBrowser->SetColumnParams( 3, columnParams );
		
			// column 4 - Kind
		columnParams = finderLikeDataBrowser->GetColumnParams( 4 );
		columnParams.fColumnOffset = 6;
		finderLikeDataBrowser->SetColumnParams( 4, columnParams );

			// hook up any signal / slots
			// column resizing
		finderLikeTableHeaders->FindSignal( kCP_Signal_ColumnResizing )->connect( finderLikeDataBrowser->FindSlot( kCP_Slot_ColumnResized ) );
		finderLikeTableHeaders->FindSignal( kCP_Signal_ColumnSort )->connect( finderLikeDataBrowser->FindSlot( kCP_Slot_ColumnSort ) );
					
			// fill in table with rows for the data
		PopulateTable( finderLikeDataBrowser );

			// associate our main window as the documents main window.
		SetMainWindow( window );
		
		CPLAT::CP_WindowManager::Instance()->Reposition( window, NULL, CPLAT::kCP_E_WindowPos_WindowCascadeOnMainScreen );

		window->Show();
	}
	catch(...)
	{
	}
}

//=========================================================================
//	FUNCTION:	PopulateTable
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Populates the passed in data browser from the files located in the selected
*	directory.
*
*	@param		inBrowser				The data browser object to add the data to.
*	
*	@exception	none
*
*	@return		void
*/
void CP_FinderLike_TV_DataBrowser_Doc::PopulateTable( CPLAT::CP_TV_DataBrowser *inBrowser )
{
	if( fFileSpecsDataSource != NULL && inBrowser != NULL )
	{
			// now add a row for each item in the source directory
		SInt32							numberFileSpecs = fFileSpecsDataSource->GetNumberSpecs();
		UInt32							nextID = 1;
		CPLAT::CP_TV_DataBrowserRow		*nextRow = NULL;
		CPLAT::CP_TV_DataBrowserRow		*childRow = NULL;
		UInt32							rowID = 1;
		SInt32							counter = 1;
		SInt32							colCounter = 1;
		
		for( counter = 1; counter <= numberFileSpecs; counter++ )
		{
			nextRow = CP_new CPLAT::CP_TV_DataBrowserRow( inBrowser, rowID );
			inBrowser->AddRootRow( nextRow );
			
				// set the cells data identifier for each cell in this row
			for( colCounter = 1; colCounter <= 4; colCounter++ )
			{
				CPLAT::CP_S_TV_DB_CellData			cellData = nextRow->GetCellData( colCounter );
				
				switch( colCounter )
				{
					case 1:
						cellData.fCellID = rowID + kNameDataSourceIDOffset;
						break;
					case 2:
						cellData.fCellID = rowID + kModifiedDataSourceIDOffset;
						break;
					case 3:
						cellData.fCellID = rowID + kSizeDataSourceIDOffset;
						break;
					case 4:
						cellData.fCellID = rowID + kKindDataSourceIDOffset;
						break;
				}
				
				nextRow->SetCellData( colCounter, cellData );
			}
				
			rowID++;
		}
	}
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Persistance ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	DoPostCreate
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Called when the document is created. We use this opportunity to ask the
*	user to select the directory to list the files from.
*
*	@param		inPostCreateFlags		Not used.
*	
*	@exception	none
*
*	@return		void
*/
void CP_FinderLike_TV_DataBrowser_Doc::DoPostCreate( SInt32 inPostCreateFlags )
{
	GetListingDirectory();
}

#if TARGET_OS_MAC

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Mac Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_MAC

#if TARGET_OS_WIN32

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Win Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_WIN32

#if TARGET_OS_LINUX

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Linux Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_LINUX

#if TARGET_CC_METROWERKS
#pragma mark === Construction / Destruction ==
#endif

//=========================================================================
//	FUNCTION:	CP_DataSource_FileSpecs
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Constructor.
*	
*	@exception	none
*/
CP_DataSource_FileSpecs::CP_DataSource_FileSpecs()
	: CP_DataSource()
{
}

//=========================================================================
//	FUNCTION:	~CP_DataSource_FileSpecs
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Destructor.
*	
*	@exception	none
*/
CP_DataSource_FileSpecs::~CP_DataSource_FileSpecs() throw()
{
}

#if TARGET_CC_METROWERKS
#pragma mark === Notification ==
#endif

//=========================================================================
//	FUNCTION:	GetData
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	CP_DataSource method called to get data for the passed in identifier.
*
*	@return		inID					The identifier of the data to be retrieved.
*	
*	@exception	none
*
*	@return		void
*/
CPLAT::CP_DataValueHolder CP_DataSource_FileSpecs::GetData( SInt32 inID )
{
	CPLAT::CP_DataValueHolder		returnHolder;

	if( inID >= 1 && inID < kModifiedDataSourceIDOffset )
	{
		CPLAT::CP_FileSystemSpec		fileSpec = fFileSpecsData.at( (UInt32) (inID - 1) );
		CPLAT::CP_OSIcon				fileIcon = CPLAT::CP_FileSystem::GetSmallIcon( fileSpec );
		CPLAT::CP_S_CH_IconTextItemInfo	itemInfo( fileIcon, fileSpec.GetName() );
		CPLAT::CP_DataValueHolder		tempHolder( itemInfo );
		
		returnHolder = tempHolder;
	}
	else if( inID >= kModifiedDataSourceIDOffset && inID < kSizeDataSourceIDOffset )
	{
		CPLAT::CP_FileSystemSpec		fileSpec = fFileSpecsData.at( (UInt32) ((inID - kModifiedDataSourceIDOffset) - 1) );
		CPLAT::CP_Date					modifiedDate = CPLAT::CP_FileSystem::GetModifiedDate( fileSpec );
		CPLAT::CP_String				modifiedDateString = modifiedDate.Format( CPLAT::CP_E_ShortDateFormat_NoDay );
		CPLAT::CP_Time					modifiedTime = CPLAT::CP_FileSystem::GetModifiedTime( fileSpec );
		CPLAT::CP_String				modifiedTimeString = modifiedTime.Format( "%I:%M %p" );
		CPLAT::CP_String				dateOutString = modifiedDateString + ", " + modifiedTimeString;
		CPLAT::CP_DataValueHolder		tempHolder( dateOutString );
		
		returnHolder = tempHolder;
	}
	else if( inID >= kSizeDataSourceIDOffset && inID < kKindDataSourceIDOffset )
	{
		CPLAT::CP_FileSystemSpec		fileSpec = fFileSpecsData.at( (UInt32) ((inID - kSizeDataSourceIDOffset) - 1) );
		CPLAT::CP_String				fileSizeString = CPLAT::CP_Utility::CP_SInt64ToString( CPLAT::CP_FileSystem::GetSize( fileSpec ) );
		CPLAT::CP_DataValueHolder		tempHolder( fileSizeString );
		
		returnHolder = tempHolder;
	}
	else if( inID >= kKindDataSourceIDOffset )
	{
		CPLAT::CP_FileSystemSpec		fileSpec = fFileSpecsData.at( (UInt32) ((inID - kKindDataSourceIDOffset) - 1) );
		CPLAT::FourCharCode				fileType = CPLAT::CP_FileSystem::GetType( fileSpec );
		CPLAT::CP_String				fileTypeString = CPLAT::CP_Utility::CP_FourCharCodeToString( fileType );
		CPLAT::CP_DataValueHolder		tempHolder( fileTypeString );
		
		returnHolder = tempHolder;
	}
		
	return( returnHolder );
}

#if TARGET_CC_METROWERKS
#pragma mark === FileSpecs ==
#endif

//=========================================================================
//	FUNCTION:	GetNumberSpecs
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Returns the number of file specs in the directory being used to list files
*	from.
*	
*	@exception	none
*
*	@return		SInt32					Number of files in the directory.
*/
SInt32 CP_DataSource_FileSpecs::GetNumberSpecs()
{
	return( fFileSpecsData.size() );
}

//=========================================================================
//	FUNCTION:	AddFileSpec
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Adds a file spec to the list of file specs.
*
*	@return		inSpec					The CP_FilesystemSpec to add to the
*										list of specs.
*	
*	@exception	none
*
*	@return		void
*/
void CP_DataSource_FileSpecs::AddFileSpec( const CPLAT::CP_FileSystemSpec& inSpec )
{
	fFileSpecsData.push_back( inSpec );
}

#if TARGET_CC_METROWERKS
#pragma mark === Sorting ==
#endif

//=========================================================================
//	FUNCTION:	Compare
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Compare two files based on the identifier to look up the file spec.
*
*	@return		inIDLeft				Identifier of the left file spec to lookup.
*	@return		inIDRight				Identifier of the right file spec to loopup.
*	
*	@exception	none
*
*	@return		void
*/
bool CP_DataSource_FileSpecs::Compare( SInt32 inIDLeft, SInt32 inIDRight,
	bool inSortUp )
{
	bool							result = false;
	
	if( (inIDLeft >= 1 && inIDLeft < kModifiedDataSourceIDOffset) &&
		(inIDRight >= 1 && inIDRight < kModifiedDataSourceIDOffset) )
	{
		CPLAT::CP_FileSystemSpec		fileSpecLeft = fFileSpecsData.at( (UInt32) (inIDLeft - 1) );
		CPLAT::CP_FileSystemSpec		fileSpecRight = fFileSpecsData.at( (UInt32) (inIDRight - 1) );
		
		if( inSortUp )
			result = (fileSpecLeft.GetName() < fileSpecRight.GetName() );
		else
			result = (fileSpecLeft.GetName() > fileSpecRight.GetName() );
	}
	else if( (inIDLeft >= kModifiedDataSourceIDOffset && inIDLeft < kSizeDataSourceIDOffset) &&
		(inIDRight >= kModifiedDataSourceIDOffset && inIDRight < kSizeDataSourceIDOffset) )
	{
		CPLAT::CP_FileSystemSpec		fileSpecLeft = fFileSpecsData.at( (UInt32) ((inIDLeft - kModifiedDataSourceIDOffset) - 1) );
		CPLAT::CP_FileSystemSpec		fileSpecRight = fFileSpecsData.at( (UInt32) ((inIDRight - kModifiedDataSourceIDOffset) - 1) );
		CPLAT::CP_Date					modifiedDateLeft = CPLAT::CP_FileSystem::GetModifiedDate( fileSpecLeft );
		CPLAT::CP_Date					modifiedDateRight = CPLAT::CP_FileSystem::GetModifiedDate( fileSpecRight );
		
		if( inSortUp )
			result = (modifiedDateLeft < modifiedDateRight );
		else
			result = (modifiedDateLeft > modifiedDateRight );
	}
	else if( (inIDLeft >= kSizeDataSourceIDOffset && inIDLeft < kKindDataSourceIDOffset) &&
		(inIDRight >= kSizeDataSourceIDOffset && inIDRight < kKindDataSourceIDOffset) )
	{
		CPLAT::CP_FileSystemSpec		fileSpecLeft = fFileSpecsData.at( (UInt32) ((inIDLeft - kSizeDataSourceIDOffset) - 1) );
		CPLAT::CP_FileSystemSpec		fileSpecRight = fFileSpecsData.at( (UInt32) ((inIDRight - kSizeDataSourceIDOffset) - 1) );
		SInt64							sizeLeft = CPLAT::CP_FileSystem::GetSize( fileSpecLeft );
		SInt64							sizeRight = CPLAT::CP_FileSystem::GetSize( fileSpecRight );
		
		if( inSortUp )
			result = (sizeLeft < sizeRight );
		else
			result = (sizeLeft > sizeRight );
	}
	else if( inIDLeft >= kKindDataSourceIDOffset && inIDRight >= kKindDataSourceIDOffset )
	{
		CPLAT::CP_FileSystemSpec		fileSpecLeft = fFileSpecsData.at( (UInt32) ((inIDLeft - kKindDataSourceIDOffset) - 1) );
		CPLAT::CP_FileSystemSpec		fileSpecRight = fFileSpecsData.at( (UInt32) ((inIDRight - kKindDataSourceIDOffset) - 1) );
		CPLAT::FourCharCode				fileTypeLeft = CPLAT::CP_FileSystem::GetType( fileSpecLeft );
		CPLAT::FourCharCode				fileTypeRight = CPLAT::CP_FileSystem::GetType( fileSpecRight );
		
		if( inSortUp )
			result = (fileTypeLeft < fileTypeRight );
		else
			result = (fileTypeLeft > fileTypeRight );
	}

	return( result );
}

#if TARGET_OS_MAC

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Mac Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_MAC

#if TARGET_OS_WIN32

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Win Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_WIN32

#if TARGET_OS_LINUX

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Linux Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_LINUX
