//=========================================================================
//
//	FILE:				CP_Bundle.h
//	
//						Created by Jens Miltner on 23.02.05.
//	
//	PURPOSE:			CP_PropertyList handling.
//	
//	HISTORY:			1st Version - January 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 107 $
//						Latest Checkout:  $Date: 2005-03-21 15:11:37 -0800 (Mon, 21 Mar 2005) $
//
//=========================================================================
#ifndef _H_CP_BUNDLE
#define _H_CP_BUNDLE

#include "CP_Types.h"
#include "CP_NoCopy.h"

#if TARGET_OS_MAC
	#include "CP_CFBundle.h"
#endif

#include "CP_PropertyList.h"

CPLAT_Begin_Namespace_CPLAT

class CP_Bundle
#if TARGET_OS_MAC
: protected CP_CFBundle
#endif
{
public:
	/// @name Constructor / Destructor
	//@{
	/// Default constructor.
	CP_Bundle();
	/// Copy constructor.
	CP_Bundle(const CP_Bundle& inBundle);
	/// Constructor for bundle given bundle filespec.
	CP_Bundle(const CP_FileSystemSpec& inBundleFileSpec);
	/// Constructor for bundle given bundle identifier.
	CP_Bundle(const CP_String& inBundleIdentifier);
	/// Destructor.
	virtual ~CP_Bundle() throw();
	//@}
	
	/// Get the main (usually application) bundle.
	static CP_Bundle& MainBundle();
	
	/// Get the creator signature for the bundle.
	OSType GetPackageCreator() const;
	/// Get the package type for the bundle.
	OSType GetPackageType() const;
	/// Get the bundle's location on disk.
	CP_FileSystemSpec GetBundleFileSpec() const;
	/// Locate the bundle's executable on disk.
	CP_FileSystemSpec GetBundleExecutableFileSpec() const;
	/// Get the short version string from the bundle's Info dictionary
	CP_String GetShortVersionString() const;
	/// Get the bundle version as a number.
	UInt32 GetVersionNumber() const;
	/// Get the copyright notice from the bundle's Info dictionary
	CP_String GetCopyrightString() const;
	/// Get the bundle identifier.
	CP_String GetIdentifier() const;
	
	/// Locate the resources folder inside the bundle.
	CP_FileSystemSpec LocateResourcesFolder() const;
	/// Locate the plug ins folder inside the bundle.
	CP_FileSystemSpec LocatePluginsFolder() const;
	/// Locate the shared support folder inside the bundle.
	CP_FileSystemSpec LocateSharedSupportFolder() const;
	/// Locate the contents folder inside the bundle
	CP_FileSystemSpec LocateContentsFolder() const;
	
	/// Locate a resource file within the bundle.
	CP_FileSystemSpec LocateResourceFile(const CP_String& inResourcename, const CP_String& inResourcetype) const;
	
	/// Get a value from the bundle's info dictionary
	const CP_PropertyList::Node* GetValueForDictionaryKey(const CP_String& inKey, CP_PropertyList::NodeType_t inRequestedValueType=CP_PropertyList::NodeType_Any) const;
	
	/// assignment - make this bundle refer to the same package as \a inBundle.
	CP_Bundle& operator = (const CP_Bundle& inBundle);

	// class implementation
#if TARGET_OS_WIN32
protected:
	CP_PropertyList::Dictionary *LoadInfoDictionary() const;
#endif	
	
	// class data
protected:
		
	// platform specific members
#if TARGET_OS_MAC
public:
	using CP_CFBundle::GetOSBundle;
	/// @name Constructor / Destructor
	//@{
	/// Construct with CoreFoundation BundleRef.
	CP_Bundle(CFBundleRef inBundleRef);
	//@}
#endif // TARGET_OS_MAC

#if TARGET_OS_WIN32
	CP_FileSystemSpec   fBundleContents;   //!<  keep the bundle contents directory (the one that contains the Windows directory)
	mutable CP_SmartRef<CP_PropertyList::Dictionary> fDictionary;     //!< keep the dictionary readed from info.plist
#endif
};

CPLAT_End_Namespace_CPLAT

#endif // _H_CP_BUNDLE
