//=========================================================================
//
//	FILE:				CP_ConsoleWindow.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A singleton class to handle console like output to a
//						window.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_ConsoleWindow
#define _H_CP_ConsoleWindow
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_NoCopy.h"
#include "CP_Rect.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class	CP_ConsoleWindow
*	@brief	A singleton class to handle console like output to a window.
*	
*	This class provides a way to output debugging messages to a wain that will
*	work on all supported platforms.
*/
class CP_EXPORT CP_ConsoleWindow : public CP_NoCopy
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_ConsoleWindow();
	//! Destructor.
	virtual								~CP_ConsoleWindow();
	//@}
			
	//! @name Instance
	//@{
	static	CP_ConsoleWindow			*Instance();
	//@}

	//! @name Appearance
	//@{
			void						ShowWindow( const CP_Rect& inBounds );
	//@}
	
	//! @name Output
	//@{
			void						printf( const char *formatString );
	//@}
			
protected:

private:
	//! @name Initialization
	//@{
			void						Init();
	//@}

	// class data
protected:
	static CP_ConsoleWindow				*fsInstance;				/*! Single instance of this class. */

	OSWindow							fOutputWindow;				/*! Platforms OSWindow for output. */
	OSControl							fOutputWindowTextControl;	/*! Text control to write output to. */
			
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_ConsoleWindow

