//=========================================================================
//
//	FILE:				CP_Cursor.h
//	
//						Copyright  2005. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Class for dealing with platform cursors.
//	
//	HISTORY:			1st Version - July 2005.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 267 $
//						Latest Checkout:  $Date: 2005-08-31 19:33:20 -0700 (Wed, 31 Aug 2005) $
//
//=========================================================================
#ifndef _H_CP_Cursor
#define _H_CP_Cursor
#ifndef __BUILD_LINUX__
	#pragma once
#endif

	// CPLAT_Framework
#ifdef __CP_USE_CP_FRAME__
	#include <CP_Frame/CP_Enums.h>
	#include <CP_Frame/CP_Types.h>
#else
	#include "CP_Enums.h"
	#include "CP_Object.h"
	#include "CP_Types.h"
#endif

CPLAT_Begin_Namespace_CPLAT

const bool kCP_RestoreCursor							= true;
const bool kCP_DontRestoreCursor						= false;

//!  CP_Cursor
/*!
  Class to manage the cursor.
*/
class CP_EXPORT CP_Cursor : public CP_Object
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Cursor(
												UInt32 inCursorID = 0,
												bool inRestoreCursor = kCP_DontRestoreCursor );
	//! Destructor.
	virtual								~CP_Cursor();
	//@}

	//! @name Getters
	//@{
										/*!
										*	Return the resource id of the current cursor.
										*	
										*	@exception	none
										*	
										*	@return		UInt32			The resource id of the current cursor.
										*/
	static	UInt32						CurrentID()									{ return( gCursorID ); }
	//@}
			
	//! @name Setters
	//@{
			void						Set( UInt32 inCursorID = 0 );
	//@}

	// class data
protected:
								
	static UInt32						gCursorID;					/*! Resource id of the current cursor. */
	
								
	bool								fRestoreCursor;				/*! Restore cursor to previous on destruction if true. */
	
	#if TARGET_OS_MAC
		UInt32							fSavedCursorID;				/*! Resource id of the previous cursor, the one that will be used if fRestoreCursor is true. */
		CursHandle						fCursorHandle;				/*! MacOS cursor handle of the current cursor. */
	#endif

	#if TARGET_OS_WIN32
		HCURSOR							fSavedCursor;				/*! Previous Windows HCURSOR, the one that will be used if fRestoreCursor is true. */
	#endif

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Cursor

