//=========================================================================
//
//	FILE:				CP_DataValueList.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class to hold a bunch of CP_DataValueHolder objects
//						that can be accessed by a string tag holding the name
//						of the data value.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 162 $
//						Latest Checkout:  $Date: 2005-04-28 15:05:07 -0700 (Thu, 28 Apr 2005) $
//
//=========================================================================
#ifndef _H_CP_DataValueList
#define _H_CP_DataValueList
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_DataValueHolder.h"
#include "CP_Object.h"
#include "CP_SignalSlot.h"

#if TARGET_CC_MSVC
	#pragma warning(disable : 4786)		// STL warnings
#endif

	// Standard Library
#include <map>

CPLAT_Begin_Namespace_CPLAT

typedef std::map<CP_String, CP_DataValueHolder, std::less<CP_String> >				CP_DataValue_Map;
typedef std::map<CP_String, CP_DataValueHolder, std::less<CP_String> >::iterator	CP_DataValue_Iterator;

typedef std::pair<const CP_String, const CP_DataValueHolder>						CP_DataValue_Entry;

/*!
*	@class	CP_DataValueList
*	@brief	A class to hold a bunch of CP_DataValueHolder objects that can be accessed
*			by a string tag holding the name of the data value.
*/

class CP_EXPORT CP_DataValueList : public CP_Object, public CP_SigSlot::CP_SlotObject
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_DataValueList();
										CP_DataValueList( const CP_DataValueList& inDataValueList );
	//! Destructor.
	virtual								~CP_DataValueList();
	//@}

	//! @name Data Value
	//@{
			void						AddValue(
												const CP_String& inDataKey,
												const CP_DataValueHolder& inDataValue );			
			void						SetValue(
											const CP_String& inDataKey,
											const CP_DataValueHolder& inDataValue );			

			void						RemoveValue( const CP_String& inDataKey );			

			bool						GetOptionalValue(
												const CP_String& inDataKey,
												CP_DataValueHolder& inOutDataValue );
	//@}

			void						ClearList();
			
			UInt32						GetCount()									{ return( fDataValues.size() ); }
			
	//! @name operators
	//@{
			CP_DataValueList&			operator = ( const CP_DataValueList& inDataValueList );
	//@}

protected:

	// class data
protected:
	CP_DataValue_Map					fDataValues;			/*! Holds the list of data values. */

		// Signals
	CP_DataValueSignal					fValueChangedSignal;	/*! Signal sent when the value of any item changes. */
		
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_DataValueList
