//=========================================================================
//
//	FILE:				CP_DragDrop.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class to manage a drag and drop operation.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 243 $
//						Latest Checkout:  $Date: 2005-07-24 20:18:42 -0700 (Sun, 24 Jul 2005) $
//
//=========================================================================
#ifndef _H_CP_DragDrop
#define _H_CP_DragDrop
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_DataStream.h"
#include "CP_Event.h"

CPLAT_Begin_Namespace_CPLAT

	// This list is not needed on the MacOS side
#if TARGET_OS_LINUX
	typedef std::vector<CP_DataStream>																CP_DataStream_Array;
	typedef std::vector<CP_DataStream>::iterator													CP_DataStream_Iterator;
	typedef std::vector<CP_DataStream>::reverse_iterator											CP_DataStream_ReverseIterator;
#elif TARGET_OS_WIN32
	typedef std::vector<CP_DataStream, std::allocator<CP_DataStream> >								CP_DataStream_Array;
	typedef std::vector<CP_DataStream, std::allocator<CP_DataStream> >::iterator					CP_DataStream_Iterator;
	typedef std::vector<CP_DataStream, std::allocator<CP_DataStream> >::reverse_iterator			CP_DataStream_ReverseIterator;
#endif

/*!
*	@class		CP_DragDrop
*	@brief		A class to manage a drag and drop operation.
*/
class CP_EXPORT CP_DragDrop
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_DragDrop();
										CP_DragDrop( const CP_DragDrop& inDragDrop );
	//! Destructor.
	virtual								~CP_DragDrop() throw();
	//@}
			
	//! @name Getters
	//@{
			UInt16						GetCount() const;
			CP_DataStream				GetItem( UInt16 inItemIndex );
	//@}

	//! @name Operators
	//@{
			CP_DragDrop&				operator = ( const CP_DragDrop& inDragDrop );
	//@}

	//! @name Tracking
	//@{
			void 						TrackDrag( CP_Event_Base& inEvent );
	//@}

protected:

	// class data
protected:
	
	#if TARGET_OS_MAC
		DragRef							fDragRef;			/*! MacOS drag reference for this drag. */
		CP_Event_Base					fDragDropEvent;		/*! MacOS CarbonEvent for the drag receive event. */
	#endif

	#if TARGET_OS_WIN32 || TARGET_OS_LINUX
		CP_DataStream_Array				*fDataItems;		/*! List of items in the drag. */
	#endif
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_DragDrop( DragRef inDragRef );
	//@}

	//! @name Getters
	//@{
			DragRef						GetDragRef() const								{ return( fDragRef ); }
			ItemReference				GetItemReference( UInt16 inIndex ) const;
	//@}

	//! @name Flavors
	//@{
			void						AddFlavor(
												FlavorType inFlavor,
												const void *inData,
												SInt32 inSize,
												FlavorFlags inFlags,
												ItemReference inItem );
	//@}
#endif

#if TARGET_OS_WIN32
public:
	//! @name DataStream
	//@{
			void						AddDataStream( CP_DataStream& inDataStream );
	//@}
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_DragDrop

