//=========================================================================
//
//	FILE:				CP_M_DragSource.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Class used in drag and drop to provide a drag source.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 247 $
//						Latest Checkout:  $Date: 2005-07-30 19:06:38 -0700 (Sat, 30 Jul 2005) $
//
//=========================================================================
#ifndef _H_CP_M_DragSource
#define _H_CP_M_DragSource
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_DragDrop.h"
#include "CP_Point.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

#if TARGET_OS_WIN32
		// forward class declarations - OLE support
	class CP_EXPORT CP_IDropSource;
	class CP_EXPORT CP_IDataObject;
	class CP_EXPORT CP_Enumerator;

	typedef void (*tQCDCallBackFunc)(CP_IDropSource *pIDropSource, BOOL inEsc, DWORD grfKeyState );
#endif

/*!
*	@class		CP_M_DragSource
*	@brief		Class used in drag and drop to provide a data source.
*/
class CP_EXPORT CP_M_DragSource
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_M_DragSource();
	//! Destructor.
	virtual								~CP_M_DragSource() throw();
	//@}

protected:
	//! @name Flavor
	//@{
	virtual	void						AddFlavor(
												CP_DragDrop& inDragDrop,
												FlavorType inFlavor,
												const void *inData,
												SInt32 inSize,
												UInt32 inFlags = 0,
												UInt32 inItem = 0 );
	//@}
				
	//! @name Tracking
	//@{
	virtual	void						TrackDrag(
												CP_DragDrop& inDragDrop,
												CP_Event_Base& inEvent );
	//@}

protected:

	// class data
protected:

	#if TARGET_OS_WIN32
		CP_IDropSource					*fIDropSource;		/*! Windows OLE IDropSource interface object. */
		CP_IDataObject					*fIDataObject;		/*! Windows OLE IDataObject interface object. */

		FORMATETC						*fDataFormats;		/*! Array of FORMATETC structure with desired formats. */
		DWORD							fNumberFormats;		/*! Number of FORMATETC structures. */

		CP_Enumerator					*fEnumerator;		/*! CP_Enumerator object. */
		
		CP_DataStream					fData;				/*! Contains the data. */
	#endif

	#if TARGET_OS_LINUX
		CP_DataStream					fData;				/*! Contains the data. */
	#endif	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
public:
	//! @name Constructor / Destructor
	//@{
										CP_M_DragSource(
												FORMATETC *inDesiredFormats,
												DWORD inNumberFormats );
	//@}

	//! @name Getters
	//@{
										/*! Return the CP_IDropSource object for the Windows OLE IDropSource interface . */
			CP_IDropSource				*GetIDropSource()				{ return( fIDropSource ); }
										/*! Return the CP_IDataObject object for the Windows OLE IDataObject interface . */
			CP_IDataObject				*GetIDataObject()				{ return( fIDataObject ); }
	//@}

	//! @name OLE interface
	//@{
			HRESULT						GetData(
												LPFORMATETC lpfe,
												LPSTGMEDIUM lpstg );

			HRESULT						GetDataHere(
												LPFORMATETC lpfe,
												LPSTGMEDIUM lpst );

			HRESULT						EnumFormatEtc(
												DWORD dwDir,
												LPENUMFORMATETC *lpEnum );
			HRESULT						GetCanonicalFormatEtc(
												LPFORMATETC lpout,
												LPFORMATETC lpin );

			HRESULT						QueryGetData( LPFORMATETC lpfe );

			HRESULT						SetData(
												LPFORMATETC lpfe,
												STGMEDIUM *lpst,
												BOOL fRelease );
			HRESULT						DAdvise(
												FORMATETC *lpfe,
												DWORD dw,
												LPADVISESINK lpas, 
												DWORD FAR *lpdw );
			HRESULT						DUnadvise( DWORD dw );
			HRESULT						EnumDAdvise( LPENUMSTATDATA *lpen );
	//@}
												
			void						ClearFlavors();
			
protected:
	//! @name OLE interface
	//@{
	virtual	HRESULT						OnGetData(
												LPFORMATETC lpfe,
												LPSTGMEDIUM lpstg );

	virtual	HRESULT						OnQueryGetData( LPFORMATETC lpfe );

	virtual	HRESULT						OnGetDataHere(
												LPFORMATETC lpfe,
												LPSTGMEDIUM lpst );
	virtual	HRESULT						OnGetCanonicalFormatEtc(
												LPFORMATETC lpout,
												LPFORMATETC lpin );
	virtual	HRESULT						OnSetData(
												LPFORMATETC lpfe,
												STGMEDIUM *lpst,
												BOOL fRelease );
	virtual	HRESULT						OnDAdvise(
												FORMATETC *lpfe,
												DWORD dw,
												LPADVISESINK lpas, 
												DWORD FAR *lpdw );
	virtual	HRESULT						OnDUnadvise( DWORD dw );
	virtual	HRESULT						OnEnumDAdvise( LPENUMSTATDATA *lpen );
	//@}
#endif

#if TARGET_OS_LINUX
#endif
};

#if TARGET_OS_WIN32

	// Helper class to communicate between the OLE IDropSource object and a CPLAT object.
//!  CP_IDropSource
/*!
  Helper class to communicate between the OLE IDropSource object and a CPLAT object.
*/
class CP_EXPORT CP_IDropSource : public IDropSource
{
protected:
        UInt32                	fRefCount;			/*! Ref count for the object. */

public:
	//! @name Constructor / Destructor
	//@{
        								CP_IDropSource();
	//! Destructor.
        								~CP_IDropSource();
	//@}

	//! @name IDropSource interface members
	//@{
	STDMETHODIMP						QueryInterface(REFIID, LPVOID FAR *);
	STDMETHODIMP_(UInt32)				AddRef();
	STDMETHODIMP_(UInt32)				Release();

	STDMETHODIMP						QueryContinueDrag(BOOL, DWORD);
	STDMETHODIMP						GiveFeedback(DWORD);
	STDMETHODIMP						SetQueryContinueDragCallBack( tQCDCallBackFunc ); 
	//@}

	// class data
private:
	tQCDCallBackFunc		m_QCDfunc;
};

	// Helper class to communicate between the OLE IDataObject object and a CPLAT object.
//!  CP_IDataObject
/*!
  Helper class to communicate between the OLE IDataObject object and a CPLAT object.
*/
class CP_EXPORT CP_IDataObject : public IDataObject
{
public:
	UInt32               		fRefCount; 			/*! Rect count for the object. */
	CP_M_DragSource				*fDragSource;		/*! CP_M_DragSource object. */
        
public:
	//! @name Constructor / Destructor
	//@{
        								CP_IDataObject( CP_M_DragSource *inDragSource );
	//! Destructor.
										~CP_IDataObject();
	//@}

	//! @name IDropTarget interface members
	//@{
	STDMETHODIMP						QueryInterface( REFIID, LPVOID FAR * );
	STDMETHODIMP_(UInt32)				AddRef();
	STDMETHODIMP_(UInt32)				Release();

	STDMETHODIMP						GetData( LPFORMATETC, LPSTGMEDIUM );
	STDMETHODIMP						GetDataHere( LPFORMATETC, LPSTGMEDIUM );
	STDMETHODIMP						QueryGetData( LPFORMATETC );
	STDMETHODIMP						GetCanonicalFormatEtc( LPFORMATETC, LPFORMATETC );
	STDMETHODIMP						SetData( LPFORMATETC, STGMEDIUM FAR *, BOOL );
	STDMETHODIMP						EnumFormatEtc( DWORD, LPENUMFORMATETC FAR * );

	STDMETHODIMP						DAdvise( FORMATETC FAR *, DWORD, LPADVISESINK, DWORD FAR *);
	STDMETHODIMP						DUnadvise( DWORD );
	STDMETHODIMP						EnumDAdvise( LPENUMSTATDATA FAR * );  
	//@}
};

	// Implementation of IEnumerator interface that provides an OLE enumerator for any
	// type of structure.
	// Used with above CP_IDataObject::EnumFormatEtc method
interface IEnum: public IUnknown
{
public:
	virtual HRESULT __stdcall			Next( UInt32, void *, UInt32  *) = 0;
	virtual HRESULT __stdcall			Skip( UInt32) = 0;
	virtual HRESULT __stdcall			Reset() = 0;
	virtual HRESULT __stdcall			Clone( IEnum ** ) = 0;
};

//!  CP_Enumerator
/*!
   Implementation of IEnumerator interface that provides an OLE enumerator for any type of structure.
   Used with above CP_IDataObject::EnumFormatEtc method.
*/
class CP_EXPORT CP_Enumerator : IEnum
{
public:
        UInt32           		fRefCount;				/*! Enumerator reference count. */
								
        LPUNKNOWN       		fpUnkOuter;				/*! IUnknown of object that created us. */
        UInt32           		fnCurrentElement;		/*! Current element. */
        UInt32           		fCount;					/*! Nubmer of FORMATETCs in us. */
        char 					*fpData;				/*! Source of FORMATECs. */
		IID						fiid;
		DWORD					fSizeEach;

	//! @name Constructor / Destructor
	//@{
										CP_Enumerator(
												LPUNKNOWN,
												void *,
												DWORD,
												DWORD,
												IID );
  	//! Destructor.
      									~CP_Enumerator();
	//@}

	//! @name IUnknown members
	//@{
	STDMETHODIMP						QueryInterface( REFIID, LPVOID * );
	STDMETHODIMP_(UInt32)				AddRef();
	STDMETHODIMP_(UInt32)				Release();
	//@}

	//! @name IEnumFORMATETC members
	//@{
	STDMETHODIMP						Next( UInt32, void *, UInt32  * );
	STDMETHODIMP						Skip( UInt32 );
	STDMETHODIMP						Reset( void );
	STDMETHODIMP						Clone( IEnum ** );
	//@}
};
#endif	// TARGET_OS_WIN32

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_M_DragSource

