//=========================================================================
//
//	FILE:				CP_MenuBar.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Class to handle management of a single top level menu bar
//						in a CPLAT application.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 247 $
//						Latest Checkout:  $Date: 2005-07-30 19:06:38 -0700 (Sat, 30 Jul 2005) $
//
//=========================================================================
#ifndef _H_CP_MenuBar
#define _H_CP_MenuBar
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Menu.h"
#include "CP_NoCopy.h"
#include "CP_STLArrayDefines.h"
#include "CP_Window.h"

#if TARGET_OS_LINUX
	#include "CP_FrameworkStructures_Linux.h"
#endif

	// Standard library
#include <vector>

CPLAT_Begin_Namespace_CPLAT

//=========================================================================
//	CP_MenuList
//=========================================================================
/*!
*	Standard library vector typedef to hold a list of each CP_Menu currently
*	installed in this menu bar.
*/
#ifndef TARGET_OS_LINUX
	typedef std::vector<CP_Menu *, std::allocator<CP_Menu *> >						CP_Menu_List;
	typedef std::vector<CP_Menu *, std::allocator<CP_Menu *> >::iterator			CP_Menu_Iterator;
#else
	typedef std::vector<CP_Menu * >													CP_Menu_List;
	typedef std::vector<CP_Menu * >::iterator										CP_Menu_Iterator;
#endif


/*!
*	@class	CP_MenuBar
*	@brief	A class to represent the top level menu bar in a CPLAT application.
*
*	This class is used in conjunction with the CP_MenuBarManager class to manage
*	the top level menu bars in your application.
*/
class CP_EXPORT CP_MenuBar : public CP_NoCopy
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_MenuBar();
										CP_MenuBar( SInt16 inMenuBarID );
	//! Destructor.
	virtual								~CP_MenuBar();
	//@}
			
	//! @name Getters
	//@{
										/*!
										*	Return the platforms menu bar reference.
										*	
										*	@return	OSMenuBar		Platforms menubar reference.
										*/
			OSMenuBar					GetOSMenuBar()								{ return( fMenuBar ); }

			SInt16						GetMenuBarID()								{ return( fMenuBarID ); }
	//@}

	//! @name Menus
	//@{
			void						AppendMenu(
												CP_Menu *inMenu,
												bool inInsertInMenuBar = true );
			void 						InsertMenu(
												CP_Menu *inMenu,
												SInt16 inBeforeMenuID );
			void						InsertMenuAt(
												CP_Menu *inMenu,
												SInt16 inPosition );

			SInt16						GetMenuIndex( SInt16 inMenuID ) const;
			SInt16						GetMenuIDAt( SInt16 inIndex );
			CP_Menu 					*GetCPMenu( SInt16 inMenuID );
			CP_Menu 					*GetCPMenuAt( SInt16 inIndex );
			CP_Menu 					*FindMenu( OSMenu inOSMenu );
	//@}

	//! @name Utility
	//@{
			void						Refresh();

			void						SetMainWindow( CP_Window *inMainWindow );

			void						AddMenuHier( CP_Menu *inMenu );

			void						LoadMenuBarMenus( SInt16 inMenuBarID );
	//@}

protected:

private:
	//! @name Menu List
	//@{
			void						AddMenuToList( CP_Menu *inMenu );
	//@}

	// class data
protected:
	CP_Menu_List						*fMenus;		/*! List of menus in this menu bar. */

	SInt16								fMenuBarID;		/*! ID of the menu bar. */
	
	OSWindow 							fOwnerWindow;	/* ! Platform window that this menu bar is attached to,
															if the platform supports this (Windows / Linux ) */

	OSMenuBar							fMenuBar;		/*! Platforms menu bar resource. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name Getters
	//@{
			CP_CommandNo				GetCommandNo( HICommand inHICommand );
	//@}

protected:
	//! @name MenuBar
	//@{
			OSMenuBar					LoadMenuBar( SInt16 inMenuBarID );
	//@}
#endif

#if TARGET_OS_WIN32
public:
	//! @name Owner Window
	//@{
			void						UpdateCommandStatus(
												OSMenu inMenu,
												CP_M_CommandHandler *inCommandHandler );
	//@}

	//! @name Owner Window
	//@{
			void						SetOwnerWindow( OSWindow inWindow );
	//@}
						
protected:
	//! @name MenuBar
	//@{
			OSMenuBar					LoadMenuBar( SInt16 inMenuBarID );
	//@}
#endif

#if TARGET_OS_LINUX
public:
	//! @name Setters
	//@{
			void						SetOwnerWindow( CP_Window *inWindow );
			void						SetOSMenuBar( OSMenuBar inMenuBar )				{ fMenuBar = inMenuBar; }
	//@}

protected:
	//! @name MenuBar
	//@{
			OSMenuBar					LoadMenuBar( SInt16 inMenuBarID );
	//@}

private:
	//! @name Menus
	//@{
			void						LoadMenus( CP_S_MenuItems_Linux *inMenuBar );
	//@}

#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_MenuBar
