/*
=========================================================================

	FILE:		CP_Pattern.h
	
				Copyright  1998-2003. Ken Stahlman. All Rights Reserved.
	
	PURPOSE:	class for dealing with platform pattern.
	
	HISTORY:	1st Version - July 1999.
	
	NOTES:		- none -

	VERSION:	Current Revision: $Revision: 2 $
				Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $

=========================================================================
*/
#ifndef _H_CP_Pattern
#define _H_CP_Pattern
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Canvas.h"
#include "CP_Object.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

	// forward class declarations
class CP_EXPORT CP_Palette;

//!  CP_Pattern
/*!
  class dealing with a platforms pattern.
*/
class CP_EXPORT CP_Pattern
{
public:
										// Construction / Destruction
									CP_Pattern();
									CP_Pattern( const CP_Pattern& inPattern );
									CP_Pattern( SInt16 inPatternID );
	virtual							~CP_Pattern();

										// Math
			bool					operator == ( const CP_Pattern& inColor ) const;
			bool					operator != ( const CP_Pattern& inColor ) const;

			CP_Pattern&				operator = ( const CP_Pattern& inColor );

	// class data
protected:
	
#if TARGET_OS_MAC
									/*! A MacOS black and white pattern. */
	Pattern							fMacOSPattern;
									/*! A MacOS color pattern. */
	PixPatHandle					fMacOSPatternHandle;
#endif
#if TARGET_OS_WIN32
									/*! A Windows pattern brush containing the pattern. */
	HBRUSH							fPatternBrush;
#endif

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
										// Construction
									CP_Pattern( Pattern *inPattern );

										// Getters
									/*! Return the MacOS black and white pattern. */
			Pattern					*GetOSPattern()							{ return( &fMacOSPattern ); }
									/*! Return the MacOS color pattern. */
			PixPatHandle			GetPixPatHandle()						{ return( fMacOSPatternHandle ); }
			
			void					LoadPattern(
										SInt16 inID,
										SInt16 inIndex );

										// Setters
			void					Set( Pattern *inPattern );
#endif

#if TARGET_OS_WIN32
public:
									/*! Return a Windows HBRUSH containing the pattern. */
			HBRUSH					GetOSPattern()							{ return( fPatternBrush ); }
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Pattern


