//=========================================================================
//
//	FILE:				CP_PropertyList.h
//	
//						Created by Jens Miltner on 01.12.04.
//	
//	PURPOSE:			CP_PropertyList handling.
//	
//	HISTORY:			1st Version - January 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 107 $
//						Latest Checkout:  $Date: 2005-03-21 15:11:37 -0800 (Mon, 21 Mar 2005) $
//
//=========================================================================
#ifndef _H_CP_PropertyList
#define _H_CP_PropertyList

#include "CP_PropertyList_Node.h"
#include "CP_PropertyList_String.h"
#include "CP_PropertyList_Number.h"
#include "CP_PropertyList_Boolean.h"
#include "CP_PropertyList_DateTime.h"
#include "CP_PropertyList_BinaryData.h"
#include "CP_PropertyList_Array.h"
#include "CP_PropertyList_Dictionary.h"

CPLAT_Begin_Namespace_CPLAT

namespace CP_PropertyList {

	/*!
	 *	Read in and parse the passed in property list file. This will open and read in the
	 *	XML property list document creating the root node specified in the document including 
	 *  all of it's child nodes.
	 *
	 *	@param		inPropertyListFile		A CP_FileSystemSpec to the property list file to be parsed.
	 *	
	 *	@exception	CP_OSErrorException raised when problems occur while opening, reading or parsing.
	 *
	 *	@return		A pointer to the root node of the property list.
	 */
	Node* ReadFromFile(const CP_FileSystemSpec& inPropertyListFile, NodeType_t inExpectedNodeType=NodeType_Any);
	
	/*!
	 *	Save the specified property list as XML into the target file. If the file exists, it'll be
	 *  overwritten.
	 *
	 *	@param		inPropertyList		The property list (it's root node) to be written.
	 *	@param		inTargetFile		A CP_FileSystemSpec to the location where the target file should be created.
	 *	
	 *	@exception	CP_OSErrorException raised when problems occur while opening, reading or parsing.
	 *
	 *	@return		void
	 */
	void WriteToFile(const Node& inPropertyList, const CP_FileSystemSpec& inTargetFile);
	
} // end of namespace CP_PropertyList

CPLAT_End_Namespace_CPLAT

#endif // _H_CP_PropertyList
