//=========================================================================
//
//	FILE:				CP_SignalSlotConnections.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class that maintains information about a signal and
//						slot connection.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 245 $
//						Latest Checkout:  $Date: 2005-07-27 23:36:26 -0700 (Wed, 27 Jul 2005) $
//
//=========================================================================
#ifndef _H_CP_SignalSlotConnections
#define _H_CP_SignalSlotConnections
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_NoCopy.h"
#include "CP_OSTypes.h"
#include "CP_String.h"
#include "CP_Types.h"

	// Standard Library
#include <vector>

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class	CP_SignalSlotConnection
*	@brief	A class that maintains information about a signal and slot connection
*/

class CP_EXPORT CP_SignalSlotConnection
{
public:
											// Construction / destruction
										CP_SignalSlotConnection();
										CP_SignalSlotConnection( const CP_SignalSlotConnection& inConnection );
	virtual								~CP_SignalSlotConnection();
			
											// Getters
			CP_String					GetSignalName()												{ return( fSignalName ); }
			UInt32						GetSignalObjectID()											{ return( fSignalObjectID ); }
			CP_String					GetSlotName()												{ return( fSlotName ); }
			UInt32						GetSlotObjectID()											{ return( fSlotObjectID ); }

											// operators
	CP_SignalSlotConnection&			operator = ( const CP_SignalSlotConnection& inConnection );

											// Setters
			void						SetSignalName( const CP_String& inSignalName )				{ fSignalName = inSignalName; }
			void						SetSignalObjectID( UInt32 inObjectID )						{ fSignalObjectID = inObjectID; }
			void						SetSlotName( const CP_String& inSlotName )					{ fSlotName = inSlotName; }
			void						SetSlotObjectID( UInt32 inObjectID )						{ fSlotObjectID = inObjectID; }

protected:

	// class data
protected:
	CP_String							fSignalName;
	UInt32								fSignalObjectID;
	CP_String							fSlotName;
	UInt32								fSlotObjectID;
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

typedef std::vector<CP_SignalSlotConnection, std::allocator<CP_SignalSlotConnection> >								CP_SignalSlotConnection_Array;
typedef std::vector<CP_SignalSlotConnection, std::allocator<CP_SignalSlotConnection> >::iterator					CP_SignalSlotConnection_Iterator;
typedef std::vector<CP_SignalSlotConnection, std::allocator<CP_SignalSlotConnection> >::reverse_iterator			CP_SignalSlotConnection_ReverseIterator;

class CP_EXPORT CP_SignalSlotConnections : public CP_NoCopy
{
public:
											// Construction / destruction
										CP_SignalSlotConnections();
	virtual								~CP_SignalSlotConnections();
			
			UInt32						GetCount();
			void						Append( CP_SignalSlotConnection& inSignalSlotConnection );
			CP_SignalSlotConnection		GetNthItem( UInt32 inIndex );
			
protected:

	// class data
protected:
	CP_SignalSlotConnection_Array		fSignalSlotConnections;
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_SignalSlotConnections

