//=========================================================================
//
//	FILE:				CP_TextBlock.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class used to manage and draw a block of text.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 45 $
//						Latest Checkout:  $Date: 2005-01-03 23:14:36 -0800 (Mon, 03 Jan 2005) $
//
//=========================================================================
#ifndef _H_CP_TextBlock
#define _H_CP_TextBlock
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Canvas.h"
#include "CP_FontMetrics.h"
#include "CP_Rect.h"
#include "CP_STLArrayDefines.h"
#include "CP_String.h"

	// Standard Library
#include <vector>

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_TextBlock
*	@brief		A class used to manage and draw a block of text. 
*	
*				This class is used with the CP_View classes to handle the
*				layout and drawing of text blocks.
*/
class CP_EXPORT CP_TextBlock
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_TextBlock(
												CP_Canvas& inDestCanvas,
												const CP_String& inText,
												const CP_Rect& inBounds,
												SInt16 inLineSpacing = -1 );
										CP_TextBlock( const CP_TextBlock& inTextBlock );
	//! Destructor.
	virtual								~CP_TextBlock() throw();
	//@}
			
	//! @name Drawing
	//@{
			void						DrawText(
												SInt16 inHorJustification,
												SInt16 inVerJustification );
	//@}

	//! @name Getters
	//@{
										/*!
										*	Returns the distance between each line when drawing
										*	the text block. This dimenstion is also referred to
										*	as the leading between lines.
										*	
										*	@exception	none
										*	
										*	@return		SInt16		The leading or distance between each line.
										*/
			SInt16						GetLineSpacing()							{ return( fLineSpacing ); }
			SInt32						GetLongestLineLength();
			SInt32						GetActualTextDepth();
	//@}

	//! @name Setters
	//@{
										/*!
										*	Sets the distance between each line when drawing
										*	the text block. This dimenstion is also referred to
										*	as the leading between lines.
										*	
										*	@exception	none
										*	
										*	@return		SInt16		The leading or distance between each line.
										*/
			void						SetLineSpacing( SInt16 inLineSpacing )		{ fLineSpacing = inLineSpacing; }
	//@}

protected:

private:
	//! @name Calculation
	//@{
			void						CalcLineEndings();
			SInt32						CalcVerStartPosn(
												SInt16 inVerJustification,
												SInt16 inLineHeight,
												SInt16 inLeading,
												SInt32 inNumberLines,
												SInt32 inAscentHeight );
			SInt32						CalcHorStartPosn(
												SInt16 inHorJustification,
												char *inText,
												SInt32 inLineStart,
												SInt32 inlineLength );

			bool						IsWordBreakChar( char inCharacter );
	//@}

	// class data
protected:
	CP_Canvas							fDestCanvas;				/*! CP_Canvas to draw the text into. */
	CP_String							fText;						/*! The text being drawn. */
	CP_Rect								fBounds;					/*! The area we want to draw the text into. */

	CP_SInt32s_Array					fLineStarts;				/*! Character index of where each line starts in the text. */
	CP_SInt32s_Array					fLineLengths;				/*! Length of each line in characters. */
	
	SInt16								fLineSpacing;				/*! Leading or distance between each line when drawn. */

	#if TARGET_OS_LINUX
		GdkFont							*fCanvasGdkFont;			/*! GdkFont used when drawing/measuring the Linux/Gtk version. */
	#endif

	CP_FontMetrics						*fDestCanvasFontMetrics;	/*! Provides information about the font used in the destination canvas. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
private:
	//! @name Drawing
	//@{
			void						DrawLine(
												SInt32 inHorPosn,
												SInt32 inVerPosn,
												char *inText,
												SInt32 inTextStart,
												SInt32 inLineLength );
	//@}
#endif	

#if TARGET_OS_WIN32
public:
	//! @name Drawing
	//@{
			void						DrawTrunc(
												SInt16 inHorAlign = CP_E_TextAlign_HorLeft,
												SInt16 inVerAlign = CP_E_TextAlign_VerTop );
	//@}

private:
	//! @name Drawing
	//@{
			void						DrawLine(
												SInt32 inHorPosn,
												SInt32 inVerPosn,
												char *inText,
												SInt32 inTextStart,
												SInt32 inLineLength );
	//@}
#endif

#if TARGET_OS_LINUX
private:
	//! @name Drawing
	//@{
			void						DrawLine(
												SInt32 inHorPosn,
												SInt32 inVerPosn,
												char *inText,
												SInt32 inTextStart,
												SInt32 inLineLength );
	//@}
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_TextBlock

