//=========================================================================
//
//	FILE:				CP_Enums.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Various enumerations used in the CPLAT framework.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 251 $
//						Latest Checkout:  $Date: 2005-08-22 08:58:47 -0700 (Mon, 22 Aug 2005) $
//
//=========================================================================
#ifndef _H_CP_Enums
#define _H_CP_Enums
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Types.h"

#if TARGET_OS_LINUX
	#include <fcntl.h>
	#include <stdio.h>
#endif

CPLAT_Begin_Namespace_CPLAT

//========================================================================
// Application running status
//========================================================================
//! Application status flags.
/*! These flags indicate the current status of a running application. */
enum CP_E_AppStatus
{
	kCP_E_AppStatus_Unknown,		/*!< Application has not yet started up */
	kCP_E_AppStatus_Startup,		/*!< Application is starting up, has not yet entered main event loop. */
	kCP_E_AppStatus_Running,		/*!< Application is running the main event loop. */
	kCP_E_AppStatus_Quitting,		/*!< Application is quiting. Caused by the user issuing a Quit command, but still open documents. */
	kCP_E_AppStatus_Quit,			/*!< Application is quit. Caused by the user issuing a Quit command. */
	kCP_E_AppStatus_UnableToRun		/*!< Something has caused the appliction startup/runtime environment check to fail. */
};
	
//========================================================================
// Events
//========================================================================
enum
{
	CP_E_CP_Event = CP_FOUR_CHAR_CODE('CPEV')
};

enum
{
	CP_E_CP_Event_SaveDocument = CP_FOUR_CHAR_CODE('SAVD')
};

//========================================================================
// File
//========================================================================
enum CP_E_NavDialogResult
{
	CP_E_NavDialogResultNone  = 0,
	CP_E_NavDialogResultCancel = 1,
	CP_E_NavDialogResultOpen = 2,
	CP_E_NavDialogResultSaveAs = 3,
	CP_E_NavDialogResultChoose = 4,
	CP_E_NavDialogResultNewFolder = 5,
	CP_E_NavDialogResultSaveChanges = 6,
	CP_E_NavDialogResultDontSaveChanges = 7,
	CP_E_NavDialogResultDiscardChanges = 8,
	CP_E_NavDialogResultReviewDocuments = 9,
	CP_E_NavDialogResultDiscardDocuments = 10
};


#if TARGET_OS_MAC
	//! CP_File permission flags.
	/*! These flags are used in the creation of a file to indicate what type of read
		write permission they will have in the underlying OS. */
	enum
	{
		CP_E_FilePerm_ReadOnly = fsRdPerm,				/*!< File can only be read from */
		CP_E_FilePerm_WriteOnly = fsWrPerm,				/*!< File can only be written to */
		CP_E_FilePerm_ReadWrite = fsRdWrPerm,			/*!< File can be read from and written to */
		CP_E_FilePerm_ReadWriteShared = fsRdWrShPerm	/*!< File can be read from and written to on a shared volume */
	};

	enum
	{
		CP_E_FileSeek_FromBegin = fsFromStart,
		CP_E_FileSeek_FromEnd = fsFromLEOF,
		CP_E_FileSeek_FromCurrent = fsFromMark
	};
#endif

#if TARGET_OS_WIN32
	enum
	{
		CP_E_FilePerm_ReadOnly = GENERIC_READ,
		CP_E_FilePerm_WriteOnly = GENERIC_WRITE,
		CP_E_FilePerm_ReadWrite = GENERIC_READ | GENERIC_WRITE,
		CP_E_FilePerm_ReadWriteShared = GENERIC_READ | GENERIC_WRITE
	};

	enum
	{
		CP_E_FileSeek_FromBegin = FILE_BEGIN,
		CP_E_FileSeek_FromEnd = FILE_END,
		CP_E_FileSeek_FromCurrent = FILE_CURRENT
	};
#endif

#if TARGET_OS_LINUX
	enum
	{
		CP_E_FilePerm_ReadOnly = O_RDONLY,
		CP_E_FilePerm_WriteOnly = O_WRONLY,
		CP_E_FilePerm_ReadWrite = O_RDWR,
		CP_E_FilePerm_ReadWriteShared = O_RDWR
	};

	enum
	{
		CP_E_FileSeek_FromBegin = SEEK_SET,
		CP_E_FileSeek_FromEnd = SEEK_END,
		CP_E_FileSeek_FromCurrent = SEEK_CUR
	};
#endif

//========================================================================
// Geometry
//========================================================================
enum CP_VHSelector
{
	vSel,
	hSel
};

//========================================================================
// Graphics
//========================================================================
enum CP_FontStyle
{
	CP_E_FontStyle_Normal = 0,
	CP_E_FontStyle_Bold = 1,
	CP_E_FontStyle_Italic = 2,
	CP_E_FontStyle_Underline = 4,
	CP_E_FontStyle_Outline = 8,
	CP_E_FontStyle_Shadow = 0x10,
	CP_E_FontStyle_Condense = 0x20,
	CP_E_FontStyle_Extend = 0x40,
	CP_E_FontStyle_BoldItalic = 0x80
};

#if TARGET_OS_MAC
	enum CP_E_TextMode
	{
		CP_E_TextMode_Transparent = srcOr,
		CP_E_TextMode_Opaque = srcCopy
	};
	
	enum CP_E_TransferMode
	{
		CP_E_TransferMode_SrcCopy = srcCopy,
		CP_E_TransferMode_SrcOr = srcOr
	};
#endif

#if TARGET_OS_WIN32
	enum CP_E_TextMode
	{
		CP_E_TextMode_Transparent = TRANSPARENT,
		CP_E_TextMode_Opaque = OPAQUE
	};

	enum CP_E_TransferMode
	{
		CP_E_TransferMode_SrcCopy = SRCCOPY,
		CP_E_TransferMode_SrcOr = SRCPAINT
	};
#endif

#if TARGET_OS_LINUX
	enum CP_E_TextMode
	{
		CP_E_TextMode_Transparent = 1,
		CP_E_TextMode_Opaque = 2
	};

	enum CP_E_TransferMode
	{
		CP_E_TransferMode_SrcCopy = 1,
		CP_E_TransferMode_SrcOr = 2
	};
#endif

//========================================================================
// Menus
//========================================================================
enum {
	kCP_E_MenuCheckMark = 0x0012,		// This corresponds to a checkmark in the keyboard font
	kCP_E_MenuDashMark = 0x002D,		// This is just a normal hyphen/minus character
	kCP_E_MenuNoMark = 0x0000			// Null character -- shows up as no mark in a menu
};

//========================================================================
// Sorting
//========================================================================
//! Sorting order flags.
/*! These flags indicate the direction a sort should occur in. */
enum CP_E_SortOrder
{
	kCP_E_SortOrder_Ascending = true,		/*!< Sorting items should be in ascending order. */
	kCP_E_SortOrder_Descending = false		/*!< Sorting items should be in descending order. */
};

//========================================================================
// UI
//========================================================================
enum CP_E_GroupBoxStyle
{
	kCP_GroupBox_Primary,
	kCP_GroupBox_Secondary
};

enum CP_E_IconStyle
{
	kCP_IconStyle_UnknownType,
	kCP_IconStyle_IconRef,
	kCP_IconStyle_IconSuite,
	kCP_IconStyle_ColorIcon,
	kCP_IconStyle_DefaultSystemType
};

enum
{
	CP_E_TextAlign_HorLeft = -2,		// teFlushLeft
	CP_E_TextAlign_HorCenter = 1,		// teCenter
	CP_E_TextAlign_HorRight = -1,		// teFlushRight
	CP_E_TextAlign_VerTop,
	CP_E_TextAlign_VerMiddle,
	CP_E_TextAlign_VerBottom
};

enum CP_E_Direction
{
	CP_E_Direction_Up,
	CP_E_Direction_Down,
	CP_E_Direction_Left,
	CP_E_Direction_Right
};

enum CP_E_RecrusiveSearch
{
	kCP_DoRecrusiveSearch = true,
	kCP_NoRecrusiveSearch = false
};

enum CP_E_WindowPositioning
{
	kCP_E_WindowPos_WindowCenterOnMainScreen     = 1,
	kCP_E_WindowPos_WindowCenterOnParentWindow   = 2,
	kCP_E_WindowPos_WindowCenterOnParentWindowScreen = 3,
	kCP_E_WindowPos_WindowCascadeOnMainScreen    = 4,
	kCP_E_WindowPos_WindowCascadeOnParentWindow  = 5,
	kCP_E_WindowPos_WindowCascadeOnParentWindowScreen = 6,
	kCP_E_WindowPos_WindowCascadeStartAtParentWindowScreen = 10,
	kCP_E_WindowPos_WindowAlertPositionOnMainScreen = 7,
	kCP_E_WindowPos_WindowAlertPositionOnParentWindow = 8,
	kCP_E_WindowPos_WindowAlertPositionOnParentWindowScreen = 9
};
	
enum CP_E_OSWindowClass
{
	kCP_E_OSWindowClass_AlertWindowClass = 1,
	kCP_E_OSWindowClass_MovableAlertWindowClass = 2,
	kCP_E_OSWindowClass_ModalWindowClass = 3,
	kCP_E_OSWindowClass_MovableModalWindowClass = 4,
	kCP_E_OSWindowClass_FloatingWindowClass = 5,
	kCP_E_OSWindowClass_DocumentWindowClass = 6,
	kCP_E_OSWindowClass_UtilityWindowClass = 8,
	kCP_E_OSWindowClass_HelpWindowClass = 10,
	kCP_E_OSWindowClass_SheetWindowClass = 11,
	kCP_E_OSWindowClass_ToolbarWindowClass = 12,
	kCP_E_OSWindowClass_PlainWindowClass = 13,
	kCP_E_OSWindowClass_OverlayWindowClass = 14,
	kCP_E_OSWindowClass_SheetAlertWindowClass = 15,
	kCP_E_OSWindowClass_AltPlainWindowClass = 16,
	kCP_E_OSWindowClass_SimpleWindowClass  = 18,
	kCP_E_OSWindowClass_DrawerWindowClass  = 20,
	kCP_E_OSWindowClass_AllWindowClasses = (unsigned long) 0xFFFFFFFF
};

#if TARGET_OS_MAC
	enum
	{
		kCP_E_ViewContent_TextOnly = kControlContentTextOnly,
		kCP_E_ViewContent_NoContent = kControlNoContent,
		kCP_E_ViewContent_IconSuiteRes = kControlContentIconSuiteRes,
		kCP_E_ViewContent_CIconRes = kControlContentCIconRes,
		kCP_E_ViewContent_PictureRes = kControlContentPictRes,
		kCP_E_ViewContent_ICONRes = kControlContentICONRes,
		kCP_E_ViewContent_IconSuiteHandle = kControlContentIconSuiteHandle,
		kCP_E_ViewContent_CIconHandle = kControlContentCIconHandle,
		kCP_E_ViewContent_PictureHandle = kControlContentPictHandle,
		kCP_E_ViewContent_IconRef = kControlContentIconRef,
		kCP_E_ViewContent_ICON = kControlContentICON,
		
		#if TARGET_API_MAC_OSX && defined(MAC_OS_X_VERSION_10_3) && (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_3)
			kCP_E_ViewContent_CGImageRef = kControlContentCGImageRef
		#else
			kCP_E_ViewContent_CGImageRef = 134
		#endif
	};

	typedef SInt16 GraphicAlignment;
	enum
	{
		kCP_E_GraphicAlign_SysDirection = kControlBevelButtonAlignSysDirection,
		kCP_E_GraphicAlign_AlignCenter = kControlBevelButtonAlignCenter,
		kCP_E_GraphicAlign_AlignLeft = kControlBevelButtonAlignLeft,
		kCP_E_GraphicAlign_AlignRight = kControlBevelButtonAlignRight,
		kCP_E_GraphicAlign_AlignTop = kControlBevelButtonAlignTop,
		kCP_E_GraphicAlign_AlignBottom = kControlBevelButtonAlignBottom,
		kCP_E_GraphicAlign_AlignTopLeft = kControlBevelButtonAlignTopLeft,
		kCP_E_GraphicAlign_AlignBottomLeft = kControlBevelButtonAlignBottomLeft,
		kCP_E_GraphicAlign_AlignTopRight = kControlBevelButtonAlignTopRight,
		kCP_E_GraphicAlign_AlignBottomRight = kControlBevelButtonAlignBottomRight
	};

	typedef SInt16 TextAlignment;
	enum
	{
		kCP_E_TextAlign_SysDirection = kControlBevelButtonAlignTextSysDirection,
		kCP_E_TextAlign_Centered = kControlBevelButtonAlignTextCenter,
		kCP_E_TextAlign_FlushRight = kControlBevelButtonAlignTextFlushRight,
		kCP_E_TextAlign_FlushLeft = kControlBevelButtonAlignTextFlushLeft
	};

	typedef SInt16 TextPlacement;
	enum
	{
		kCP_E_TextPlacement_SysDirection = kControlBevelButtonPlaceSysDirection,
		kCP_E_TextPlacement_Normal = kControlBevelButtonPlaceNormally,
		kCP_E_TextPlacement_RightOfGraphic = kControlBevelButtonPlaceToRightOfGraphic,
		kCP_E_TextPlacement_LeftOfGraphic = kControlBevelButtonPlaceToLeftOfGraphic,
		kCP_E_TextPlacement_BelowGraphic = kControlBevelButtonPlaceBelowGraphic,
		kCP_E_TextPlacement_AboveGraphic = kControlBevelButtonPlaceAboveGraphic
	};

	typedef UInt16 DisclosureTriangleOrientation;
	enum
	{
		kCP_E_DisclosureOrientation_Default = kControlDisclosureTrianglePointDefault,
		kCP_E_DisclosureOrientation_PointRight = kControlDisclosureTrianglePointRight,
		kCP_E_DisclosureOrientation_PointLeft = kControlDisclosureTrianglePointLeft
	};

	typedef UInt32 WindowModality;
	enum
	{
		kCP_E_WindowModalityNone = kWindowModalityNone,
		kCP_E_WindowModalitySystemModal = kWindowModalitySystemModal,
		kCP_E_WindowModalityAppModal = kWindowModalityAppModal,
		kCP_E_WindowModalityWindowModal = kWindowModalityWindowModal
	};

	enum CP_E_ViewFlags
	{
		CP_E_ViewFlags_None = 0x00000000,
		CP_E_ViewFlags_IsOpaque = 0x00000001,
		CP_E_ViewFlags_IsContainer = 0x00000002,
		CP_E_ViewFlags_IsScrollable = 0x00000004,
		CP_E_ViewFlags_IsCustomView = 0x0000008,
		CP_E_ViewFlags_SupportsDragAndDrop = 0x00000010,
		CP_E_ViewFlags_SupportsLiveFeedback = 0x00000020,
		CP_E_ViewFlags_NoDrawing = 0x00000040,
		CP_E_ViewFlags_SupportsMouseTracking = 0x00001000
	};

	enum kCP_E_AppearanceManagerVersion
	{
		kCP_E_AppearanceManagerVersion_1 = 0x00000100,
		kCP_E_AppearanceManagerVersion_101 = 0x00000101,
		kCP_E_AppearanceManagerVersion_11 = 0x00000110
	};

	enum kCP_E_ButtonColors
	{
		CP_E_ButtonColor_White = 0,
		CP_E_ButtonColor_Black = 1,

		CP_E_ButtonColor_Gray_1 = 2,
		CP_E_ButtonColor_Gray_2 = 3,
		CP_E_ButtonColor_Gray_3 = 4,
		CP_E_ButtonColor_Gray_4 = 5,
		CP_E_ButtonColor_Gray_5 = 6,
		CP_E_ButtonColor_Gray_6 = 7,
		CP_E_ButtonColor_Gray_7 = 8,
		CP_E_ButtonColor_Gray_8 = 9,
		CP_E_ButtonColor_Gray_9 = 10,
		CP_E_ButtonColor_Gray_10 = 11,
		CP_E_ButtonColor_Gray_11 = 12,
		CP_E_ButtonColor_Gray_12 = 13
	};

//========================================================================
// Utility / Misc
//========================================================================
	enum kCP_E_CarbonVersion
	{
		kCP_E_CarbonVersion_1 = 0x00000100,			// 1.0
		kCP_E_CarbonVersion_11 = 0x00000110,		// 1.1
		kCP_E_CarbonVersion_12 = 0x00000120,		// 1.2
		kCP_E_CarbonVersion_13 = 0x00000130,		// 1.3
		kCP_E_CarbonVersion_14 = 0x00000140,		// 1.4
		kCP_E_CarbonVersion_15 = 0x00000150,		// 1.5
		kCP_E_CarbonVersion_16 = 0x00000160			// 1.6
	};

	enum kCP_E_OSX_Version
	{
		kCP_E_OSX_Version_10 = 0x00001000,			// OSX 10.0
		kCP_E_OSX_Version_101 = 0x00001010,			// OSX 10.1
		kCP_E_OSX_Version_102 = 0x00001020,			// OSX 10.2		// Jaguar
		kCP_E_OSX_Version_103 = 0x00001030,			// OSX 10.3		// Panther
		kCP_E_OSX_Version_104 = 0x00001040			// OSX 10.4		// Tiger
	};
#endif

enum
{
	kCP_E_ClipboardFormat_DataStream = 100,
	kCP_E_ClipboardFormat_DataStreamType = CP_FOUR_CHAR_CODE('DDDT'),
	kCP_E_ClipboardFormat_CPFileSpec = 101,
	kCP_E_ClipboardFormat_CPFileSpecType = CP_FOUR_CHAR_CODE('CPFS')
};

#if TARGET_OS_WIN32 || TARGET_OS_LINUX	
	enum
	{
		kCP_E_ViewContent_TextOnly = 0,
		kCP_E_ViewContent_NoContent = 0,
		kCP_E_ViewContent_IconSuiteRes = 1,
		kCP_E_ViewContent_CIconRes = 2,
		kCP_E_ViewContent_PictureRes = 3,
		kCP_E_ViewContent_ICONRes = 4,
		kCP_E_ViewContent_IconSuiteHandle = 129,
		kCP_E_ViewContent_CIconHandle = 130,
		kCP_E_ViewContent_PictureHandle = 131,
		kCP_E_ViewContent_IconRef = 132,
		kCP_E_ViewContent_ICON = 133,
		kCP_E_ViewContent_CGImageRef = 134
	};

	typedef SInt16 GraphicAlignment;
	enum
	{
		kCP_E_GraphicAlign_SysDirection = -1,
		kCP_E_GraphicAlign_AlignCenter = 0,
		kCP_E_GraphicAlign_AlignLeft = 1,
		kCP_E_GraphicAlign_AlignRight = 2,
		kCP_E_GraphicAlign_AlignTop = 3,
		kCP_E_GraphicAlign_AlignBottom = 4,
		kCP_E_GraphicAlign_AlignTopLeft = 5,
		kCP_E_GraphicAlign_AlignBottomLeft = 6,
		kCP_E_GraphicAlign_AlignTopRight = 7,
		kCP_E_GraphicAlign_AlignBottomRight = 8
	};

	typedef SInt16 TextAlignment;
	enum
	{
		kCP_E_TextAlign_SysDirection = 0,
		kCP_E_TextAlign_Centered = 1,
		kCP_E_TextAlign_FlushRight = -1,
		kCP_E_TextAlign_FlushLeft = -2
	};

	typedef SInt16 TextPlacement;
	enum
	{
		kCP_E_TextPlacement_SysDirection = -1,
		kCP_E_TextPlacement_Normal = 0,
		kCP_E_TextPlacement_RightOfGraphic = 1,
		kCP_E_TextPlacement_LeftOfGraphic = 2,
		kCP_E_TextPlacement_BelowGraphic = 3,
		kCP_E_TextPlacement_AboveGraphic = 4
	};

	typedef UInt16 DisclosureTriangleOrientation;
	enum
	{
		kCP_E_DisclosureOrientation_Default = 0,
		kCP_E_DisclosureOrientation_PointRight = 1,
		kCP_E_DisclosureOrientation_PointLeft = 2
	};

	enum CP_E_ViewFlags
	{
		CP_E_ViewFlags_None = 0x00000000,
		CP_E_ViewFlags_IsOpaque = 0x00000001,
		CP_E_ViewFlags_IsContainer = 0x00000002,
		CP_E_ViewFlags_IsScrollable = 0x00000004,
		CP_E_ViewFlags_IsCustomView = 0x0000008,
		CP_E_ViewFlags_SupportsDragAndDrop = 0x00000010,
		CP_E_ViewFlags_SupportsLiveFeedback = 0x00000020,
		CP_E_ViewFlags_NoDrawing = 0x00000040,
		CP_E_ViewFlags_SupportsMouseTracking = 0x00001000
	};

	typedef UInt32 WindowModality;
	enum
	{
		kCP_E_WindowModalityNone = 0,
		kCP_E_WindowModalitySystemModal = 1,
		kCP_E_WindowModalityAppModal = 2,
		kCP_E_WindowModalityWindowModal = 3
	};

	#if TARGET_OS_WIN32
		enum kCP_E_ButtonColors
		{
			CP_E_ButtonColor_White = 255,
			CP_E_ButtonColor_Black = 0,

			CP_E_ButtonColor_Gray_1 = COLOR_3DLIGHT,
			CP_E_ButtonColor_Gray_2 = COLOR_3DLIGHT,
			CP_E_ButtonColor_Gray_3 = COLOR_3DLIGHT,
			CP_E_ButtonColor_Gray_4 = COLOR_BTNHIGHLIGHT,
			CP_E_ButtonColor_Gray_5 = COLOR_BTNHIGHLIGHT,
			CP_E_ButtonColor_Gray_6 = COLOR_3DSHADOW,
			CP_E_ButtonColor_Gray_7 = COLOR_3DFACE,
			CP_E_ButtonColor_Gray_8 = COLOR_3DSHADOW,
			CP_E_ButtonColor_Gray_9 = COLOR_3DSHADOW,
			CP_E_ButtonColor_Gray_10 = COLOR_3DSHADOW,
			CP_E_ButtonColor_Gray_11 = COLOR_3DDKSHADOW,
			CP_E_ButtonColor_Gray_12 = COLOR_3DDKSHADOW
		};
	#elif TARGET_OS_LINUX
		enum kCP_E_ButtonColors
		{
			CP_E_ButtonColor_White = 0,
			CP_E_ButtonColor_Black = 1,

			CP_E_ButtonColor_Gray_1 = 2,
			CP_E_ButtonColor_Gray_2 = 3,
			CP_E_ButtonColor_Gray_3 = 4,
			CP_E_ButtonColor_Gray_4 = 5,
			CP_E_ButtonColor_Gray_5 = 6,
			CP_E_ButtonColor_Gray_6 = 7,
			CP_E_ButtonColor_Gray_7 = 8,
			CP_E_ButtonColor_Gray_8 = 9,
			CP_E_ButtonColor_Gray_9 = 10,
			CP_E_ButtonColor_Gray_10 = 11,
			CP_E_ButtonColor_Gray_11 = 12,
			CP_E_ButtonColor_Gray_12 = 13
		};
	#endif
#endif

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Enums
