//=========================================================================
//
//	FILE:				CP_File.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Class to handle a file in CPLAT. All of the actual
//						implementation for a specific platform is handled in
//						this classes base class CP_File_Imp which has a corresponding
//						implementation for each supported OS.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_File
#define _H_CP_File
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_File_Imp_Mac.h"
#include "CP_File_Imp_Win.h"
#include "CP_FileSystemSpec.h"

#if TARGET_OS_LINUX
	#include "CP_File_Imp_Gtk.h"
#endif

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_File
*	@brief		A class for representing a platforms file object.
*	
*				This class is implemented for each of the supported platforms
*				through a CP_File_Imp class.
*
*				The name and location of this file object is defined by its
*				CP_FileSystemSpec.
*
*	@ingroup	FileSystem
*/
class CP_EXPORT CP_File : public CP_File_Imp
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_File();
										CP_File( const CP_FileSystemSpec& inFileSpec );
	//! Destructor.
	virtual								~CP_File();
	//@}

protected:

	// class data
protected:
		
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_File

