//=========================================================================
//
//	FILE:				CP_FileSystemDialogsReply.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Provides support for file system dialogs that return
//						a reply with a user selection.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 7 $
//						Latest Checkout:  $Date: 2004-12-23 15:45:25 -0800 (Thu, 23 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_FileSystemDialogsReply
#define _H_CP_FileSystemDialogsReply
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_FileSystemDialogsBase.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_FileSystemDialogsReply
*	@brief		Provides support for file system dialogs that return a reply with a user selection.
*/
class CP_EXPORT CP_FileSystemDialogs_Reply : public CP_FileSystemDialogs_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_FileSystemDialogs_Reply();
	//! Destructor.
	virtual								~CP_FileSystemDialogs_Reply() throw();
	//@}
			
protected:

	// class data
protected:

	#if TARGET_OS_MAC
		static NavObjectFilterUPP		fsNavObjectFilterUPP;		/*! NavigationServices object filter UPP. */
		static NavPreviewUPP			fsNavPreviewUPP;			/*! NavigationServices preview UPP. */
	#endif
			
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
protected:
	virtual	void						HandleEvent(
												NavEventCallbackMessage inSelector, 
												NavCBRec& ioParams );

	virtual	void						RunPlatformDialog();

										/*!
										*	Virtual function to handle replay. Subclasses override to
										*	do actual processing of reply.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						ProcessReply()											{}

										/*!
										*	Called when the user has done something to indicate they have
										*	made a selection, such as pressing Open or Save button. Subclasses
										*	override to do actual processing of action.
										*	
										*	@param		inSendingDialog			Not used.
										*	@param		inNavDialogResult		Not used.
										*	
										*	@exception	none
										*
										*	@return		void
										*/
	virtual	void						HandleUserAction(
												CP_FileSystemDialogs_Base */*inSendingDialog*/,
												CP_E_NavDialogResult /*inNavDialogResult*/ )	{}
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_FileSystemDialogsReply

