//=========================================================================
//
//	FILE:				CP_AE_Descriptor.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Wrapper class for dealing with Apple Event descriptors.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 266 $
//						Latest Checkout:  $Date: 2005-08-29 21:53:00 -0700 (Mon, 29 Aug 2005) $
//
//=========================================================================
#ifndef _H_CP_AE_Descriptor
#define _H_CP_AE_Descriptor
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_MAC

	// CPLAT_Framework
#include "CP_FileSystemSpec.h"
#include "CP_Point.h"
#include "CP_Rect.h"
#include "CP_String.h"

CPLAT_Begin_Namespace_CPLAT

	// defines for time descriptors
//! CP_E_TimeDescriptors
/*! Defines for various time descriptors we use. */
enum CP_E_TimeDescriptors
{
	typeDateTimeRec												= CP_FOUR_CHAR_CODE('dtim'),
	typeLongDateTimeRec											= CP_FOUR_CHAR_CODE('ldtr')
};

	// forward class declaration
class CP_AE_AppleEvent;

/*!
*	@class		CP_AE_Descriptor 
*	@brief		The CP_AE_Descriptor class is a MacOS only class that provides a wrapper for the
*				MacOS AEDesc structure. NOTE: Most of these routines will throw an CP_OSErr exception
*				if they don't work with no error.
*
*	@ingroup	AppleEvents
*/
class CP_EXPORT CP_AE_Descriptor
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_AE_Descriptor();
										CP_AE_Descriptor(
												const AEDesc& inDesc,
												AEKeyword inKeyword,
												DescType inDesiredType = typeWildCard );
										CP_AE_Descriptor( const CP_AE_Descriptor& inDescriptor );
	//! Destructor.
	virtual								~CP_AE_Descriptor();
	//@}

	//! @name Creation
	//@{
			void						Dispose();
											
			void						CreateDescriptor(
												const DescType inTypeCode,
												const Ptr inData,
												const Size inLength );
			void						MakeNULL();
	//@}

	//! @name Coercion
	//@{
			OSErr						AttemptCoercion( DescType typeToCoerceTo );
			void						CoerceInPlace( DescType typeToCoerceTo );
			CP_AE_Descriptor			Coerce( DescType typeToCoerceTo ) const;
			void						CoerceToStandardType();
	//@}

	//! @name Copiers
	//@{
			CP_AE_Descriptor			operator =( const CP_AE_Descriptor& inDescriptor );
			void						CopyDesc( const CP_AE_Descriptor& desc );
	//@}

	//! @name Getters
	//@{
										/*!
										*	Return a DescType that describes the type of descriptor this is.
										*
										*	@exception	none
										*
										*	@return		DescType const		The DescType for this descriptor.
										*/
			DescType					DescriptorType() const					{ return( fAEDesc.descriptorType ); }
			Handle						GetData();
			
			void						GetDescriptor(
												const AEDesc& inDesc,
												AEKeyword inKey,
												DescType inDesiredType = typeWildCard );
			OSErr						GetOptionalDescriptor(
												const AEDesc& inDesc,
												AEKeyword inKey,
												DescType inDesiredType = typeWildCard );
												
			void						GetNthPtr(
												SInt32 inIndex,
												DescType inDesiredType,
												AEKeyword *inAEKeyword,
												DescType *inTypeCode,
												void *inDataPtr,
												Size inMaximumSize,
												Size *inOutActualSize );
			void						GetBlock(
												Ptr inData,
												Size inLength,
												DescType inDesiredType ) const;
			SInt16						GetSInt16() const;
			SInt32						GetSInt32() const;
			bool						GetBoolean() const;
			DescType					GetDescType() const;
			AEKeyword					GetKeyword() const;
			DescType					GetEnumeration() const;
			DescType					GetOrdinal() const;
			CP_Point					GetPoint() const;
			CP_Rect						GetRect() const;
			void						GetPString( Str255 result ) const;
			CP_String					GetString() const;
			#if CP_UseDepreciated_10_4_APIs
				DateTimeRec				GetDateTimeRec() const;
			#endif	// CP_UseDepreciated_10_4_APIs
			CP_FileSystemSpec			GetFileSpec() const;

			SInt32						GetItemCount();
			OSErr						GetNthItem(
												SInt32 inItemIndex,
												DescType *inOutDescType,
												CP_AE_Descriptor *inOutItem );
	//@}
			
			operator					AEDesc *()										{ return( &fAEDesc ); }
			operator					const AEDesc *() const							{ return( &fAEDesc ); }

			operator					AEDesc&() 										{ return( fAEDesc ); }
			operator					const AEDesc&() const							{ return( fAEDesc ); }

	//! @name Setters
	//@{
			void						CopyData(
												const DescType inTypeCode,
												const Ptr inData,
												const Size inLength );
			void						MakeBoolean( const Boolean inData );
			void						MakeBoolean( const bool inData );
			void						MakeSInt16( const SInt16 inData );
			void						MakeSInt32( const SInt32 inData );
			void						MakeUInt32( const UInt32 inData );
			void						MakePoint( const CP_Point& inPoint );
			void						MakeRect( const CP_Rect& inRect );
			void						MakeString( Str255 inData );
			void						MakeString( const CP_String inData );
			#if CP_UseDepreciated_10_4_APIs
				void					MakeDateTimeRec( const DateTimeRec inDateTime );
				void					MakeLongDateTimeRec( const DateTimeRec inDateTime );
				void					MakeDateTimeRec( const SInt32 inSecsSince1904 );
				void					MakeLongDateTimeRec( const LongDateRec inDateTime );
				void					MakeLongDateTimeRec( LongDateTime inLsecs );
			#endif	// CP_UseDepreciated_10_4_APIs
			void						MakeFileSpec( const FSSpec& spec );
			void						MakeFileSpec( const CP_FileSystemSpec& inFileSpec );
			void						AdoptAlias( Handle inAlias );
			void						MakeAlias( const CP_FileSystemSpec& inFileSpec );
			void						MakeProcessSerialNumber( ProcessSerialNumber inPSN );
			void						MakeObjectSpecifier(
												DescType inDesiredClass,
												CP_AE_Descriptor inContainer,
												DescType inKeyForm,
												CP_AE_Descriptor inKeyData,
												bool inDisposeInputs );
			void						MakeCompDescriptor(
												DescType inComparisonOperator,
												DescType inPropertyIdentifier,
												CP_AE_Descriptor inCompareWith,
												bool inDisposeInputs );
			void						MakeEnumeration( const DescType inEnumeration );
			void						MakeDescType( const DescType inData );
			void						MakeKeyword( const AEKeyword inKeyword );
			void						MakeOrdinal( const DescType inData );
			void						MakeTypeOrInteger( const DescType inData );

			void						AdoptHandle(
												DescType inDataType,
												Handle inDataHandle );
			void						AdoptDesc( const AEDesc& inDescriptor );
	//@}
			
	//! @name NULL Descriptor
	//@{
										/*!
										*	Return true if this is a NULL descriptor, ie doesn't describe anything.
										*
										*	@exception	none
										*
										*	@return		bool const		True if this is a NULL desciptor.
										*/
			bool						IsNullDescriptor() const					{ return( fAEDesc.descriptorType == typeNull ); }

			bool						IsNullDataHandle() const;
	//@}

			void						swap( CP_AE_Descriptor& inOutDescriptor ) throw();

protected:

	// class data members
public:
	AEDesc								fAEDesc;
};

CPLAT_End_Namespace_CPLAT

#endif	// TARGET_OS_MAC

#endif	// _H_CP_AE_Descriptor

