//=========================================================================
//
//	FILE:				CP_Graphics_Imp_QD.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			QuickDraw implementation of the CP_Graphics object.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 226 $
//						Latest Checkout:  $Date: 2005-07-02 15:14:01 -0700 (Sat, 02 Jul 2005) $
//
//=========================================================================
#ifndef _H_CP_Graphics_Imp_QD
#define _H_CP_Graphics_Imp_QD
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_MAC && CP_UseDepreciated_10_4_APIs

	// CPLAT_Framework
#include "CP_Graphics_Imp.h"
#include "CP_StTextStateSaver.h"
#include "CP_StColorStateSaver.h"
#include "CP_StPenStateSaver.h"

CPLAT_Begin_Namespace_CPLAT

enum CP_E_PathOperation
{
	CP_E_PathOperation_NoOp,
	CP_E_PathOperation_ClosePath,
	CP_E_PathOperation_MoveTo,
	CP_E_PathOperation_LineTo,
	CP_E_PathOperation_Rect,
	CP_E_PathOperation_Oval,
	CP_E_PathOperation_Bezier,
	CP_E_PathOperation_QuadBezier,
	CP_E_PathOperation_Poly,
	CP_E_PathOperation_Arc
};

// ===================================================================================
//	CP_S_PathOperation - structure used to hold commands for constructing a path.
// ===================================================================================
struct CP_S_PathOperation
{
	CP_E_PathOperation		fPathOp;
	CP_Point				fStartPoint;
	CP_Point				fEndPoint;
	CP_Point				fControlPointOne;
	CP_Point				fControlPointTwo;
	
							CP_S_PathOperation();
							CP_S_PathOperation(
									CP_E_PathOperation inPathOp,
									CP_Point inStartPoint,
									CP_Point inEndPoint );
							CP_S_PathOperation(
									CP_E_PathOperation inPathOp,
									CP_Point inStartPoint,
									CP_Point inEndPoint,
									CP_Point inControlPointOne,
									CP_Point inControlPointTwo );
							CP_S_PathOperation( const CP_S_PathOperation& inOperation );

	CP_S_PathOperation&		operator =( const CP_S_PathOperation& inOperation );
};

	// list of path operations
typedef std::vector<CP_S_PathOperation, std::allocator<CP_S_PathOperation> >					CP_S_PathOperation_Array;
typedef std::vector<CP_S_PathOperation, std::allocator<CP_S_PathOperation> >::iterator			CP_S_PathOperation_Iterator;
typedef std::vector<CP_S_PathOperation, std::allocator<CP_S_PathOperation> >::reverse_iterator	CP_S_PathOperation_ReverseIterator;

/*!
*	@class		CP_Graphics_Imp_QD
*	@brief		CP_Graphics_Imp subclass to perform drawing using QuickDraw.
*
*				This is the CP_Graphics_Imp class used by a CP_Graphics object when the
*				user has selected QuickDraw as the underlying graphics engine. This will
*				also be selected as the native platform type when not running under MacOS
*				Mach with the Quartz drawing available.
*/
class CP_EXPORT CP_Graphics_Imp_QD : public CP_Graphics_Imp
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Graphics_Imp_QD( CP_Canvas& inDestCanvas );
	//! Destructor.
	virtual								~CP_Graphics_Imp_QD();
	//@}

	//! @name Setters
	//@{
	virtual	void						SetAlpha( float inAlpha );
	virtual	void						SetLineWidth( float lineWidth );
	virtual	void						SetStrokeColor( CP_ARGBColor& inStrokeColor );
	virtual void						SetFillColor( CP_ARGBColor& inFillColor );
	//@}
			
	//! @name Graphics State
	//@{
	virtual	void						SaveState();
	virtual	void						RestoreState();

	virtual	void						ScaleCTM(
											float inHorAmt,
											float inVerAmt );
	virtual	void						RotateCTM( float inAngle );
	virtual	void						SkewCTM(
											float inHorAmt,
											float inVerAmt );
	//@}

	//! @name Arcs
	//@{
	virtual	void						StrokeArc(
												float inHorCenter,
												float inVerCenter,
												float inRadius,
												float inStartAngle,
												float inEndAngle );
	virtual void						FillArc(
												float inHorCenter,
												float inVerCenter,
												float inRadius,
												float inStartAngle,
												float inEndAngle );
	//@}

	//! @name Bezier
	//@{
	virtual	void						StrokeBezier(
												const CP_Point& inStartPoint,
												const CP_Point& inControlPointOne,
												const CP_Point& inControlPointTwo,
												const CP_Point& inEndPoint );
	virtual	void						StrokeQuadBezier(
												const CP_Point& inStartPoint,
												const CP_Point& inControlPointOne,
												const CP_Point& inEndPoint );
	//@}

	//! @name Gradient
	//@{
	virtual	void						DrawGradient(
												CP_E_GradientType inGradientType,
												CP_Rect& inDestBounds );
	virtual	void						SetGradientAngle( SInt16 inAngle )						{ fGradientAngle = inAngle; }
	virtual	void						SetGradientColor(
												const SInt16 inIndex,
												const CP_ARGBColor& inColour );
	virtual	void						SetGradientCount( SInt16 inGradientCount )				{ fGradientCount = inGradientCount; }
	//@}

	//! @name Icons
	//@{
	virtual	void						DrawIcon(
												const CP_Rect& inBounds,
												CP_E_IconStyle inIconType,
												SInt16 inIconID );
	virtual	void						DrawIcon(
												CP_OSIcon& inPicture,
												const CP_Rect& inBounds );
	//@}

	//! @name Images
	//@{
	virtual	void						DrawImage(
												SInt16 inImageResID,
												CP_Rect& inDestBounds );
	virtual	void						DrawImage(
												const CP_FileSystemSpec& inImageSpec,
												CP_Rect& inDestBounds );
	virtual	void						DrawImage(
												const CP_FileSystemSpec& inImageSpec,
												CP_Rect& inSourceBounds,
												CP_Rect& inDestBounds );
	virtual	void						DrawPicture(
												CP_OSPicture& inPicture,
												const CP_Rect& inBounds );
	//@}

	//! @name Lines
	//@{
	virtual	void						Line(
												const CP_Point& inPoint,
												const CP_Size& inSize );
	virtual	void						Line(
												float inStartHorLocation,
												float inStartVerLocation,
												float inEndHorLocation,
												float inEndVerLocation );
	//@}

	//! @name Paths
	//@{
	virtual	void						BeginPath();
	virtual	void						ClosePath();
	virtual	void						StrokePath();
	virtual	void						FillPath();
	virtual	void						FillPath( CP_Pattern& inPattern );
	virtual	void						PathMoveTo( const CP_Point& inPoint );
	virtual	void						PathMoveTo(
												float inHorLocation,
												float inVerLocation );
	virtual	void						PathAddLineTo( const CP_Point& inPoint );
	virtual	void						PathAddLines(
												const CP_Point inPoint[],
												SInt16 inNumberPoints );
	virtual	void						PathAddLine(
												const CP_Point& inStartPoint,
												const CP_Point& inEndPoint );
	virtual	void						PathAddLine(
												float inStartH,
												float inStartV,
												float inEndH,
												float inEndV );
	virtual	void						PathAddRect( const CP_Rect& inBounds );
	virtual	void						PathAddOval( const CP_Rect& inBounds );
	virtual	void						PathAddBezier(
												const CP_Point& inStartPoint,
												const CP_Point& inControlPointOne,
												const CP_Point& inControlPointTwo,
												const CP_Point& inEndPoint );
	virtual	void						PathAddQuadBezier(
												const CP_Point& inStartPoint,
												const CP_Point& inControlPointOne,
												const CP_Point& inEndPoint );
	virtual	void						PathAddPoly( CP_Point_Array& inPoints );
	virtual	void						PathAddArc(
												float x,
												float y,
												float radius,
												float startAngle,
												float endAngle,
												SInt16 clockwise );
	virtual	void						ClipPath();
	//@}

	//! @name Polygons
	//@{
	virtual	void						StrokePoly( CP_Point_Array& inPoints );
	virtual	void						FillPoly( CP_Point_Array& inPoints );
	//@}

	//! @name Rects
	//@{
	virtual	void						StrokeRect( const CP_Rect& inBounds );
	virtual	void						EraseRect( const CP_Rect& inBounds );
	virtual	void						FillRect( const CP_Rect& inBounds );
	//@}
			
	//! @name Regions
	//@{
	virtual	void						StrokeRegion( const CP_Region& inRegion );
	virtual	void						FillRegion( const CP_Region& inRegion );
	//@}

	//! @name Round Rect
	//@{
	virtual	void						StrokeRoundRect(
												const CP_Rect& inBounds,
												SInt16 inHorRadius,
												SInt16 inVerRadius );
	virtual	void						FillRoundRect( 
												const CP_Rect& inBounds,
												SInt16 inHorRadius,
												SInt16 inVerRadius );
	//@}

	//! @name Ovals
	//@{
	virtual	void						StrokeOval( const CP_Rect& inBounds );
	virtual	void						FillOval( const CP_Rect& inBounds );
	//@}

	//! @name Text
	//@{
	virtual	void						DrawString(
												CP_FontSpec& inFontSpec,
												const CP_String& inText,
												const CP_Point& inStartPoint );
	virtual	void						DrawText(
												CP_FontSpec& inFontSpec,
												const CP_String& inText,
												const CP_Rect& inBounds,
												SInt16 inHorAlign = CP_E_TextAlign_HorLeft,
												SInt16 inVerAlign = CP_E_TextAlign_VerTop,
												bool inWrapText = true );
	//@}

protected:

private:
	//! @name Misc
	//@{
			void						InitGradient();

			RgnHandle					CreatePathRegion();

			void						HandleOffscreenTransform(
												CP_OffscreenCanvas& inSourceCanvas,
												CP_Rect& inSourceRect,
												CP_Rect& inDestRect,
												CP_OffscreenCanvas& inDestCanvas );

			void						FrameQuadCurve(
												const CP_Point& inStartPoint,
												const CP_Point& inControlPointOne,
												const CP_Point& inEndPoint );
			void						FrameCurve(
												kCP_QuadCurveCurve *inQuadCurve,
												SInt16 inLevel );

			void						CreatePolyShape();
			CP_Rect						GetPolyBounds();
	//@}
			
	// class data
protected:
	CP_S_PathOperation_Array			fPath;					/*! An array of path operations. */
	RgnHandle							fClipPath;				/*! Region used for clipping. */
	RgnHandle							fSavedClipRgn;			/*! Saved clipping region. */
	
	bool								fNeedTransform;			/*! Flag to indicate if any drawing operation needs transformation. */
	
	float								fScaleAmtHor;			/*! Horizontal scale amount. */
	float								fScaleAmtVer;			/*! Vertical scale amount. */
	
	float								fAngle;					/*! Angle to rotate drawing. */

	float								fSkewAmtHor;			/*! Horizontal skew amount. */
	float								fSkewAmtVer;			/*! Vertical skew amount. */

	CP_StTextStateSaver					fTextState;				/*! Current text state. */
	CP_StColorStateSaver				fColorState;			/*! Current color state. */
	CP_StPenStateSaver					fPenState;				/*! Current pen state. */

	PolyHandle							fPoly;					/*! Polygon handle when drawing polygons. */
	CP_Point_Array						fPoints;				/*! Points used to draw the polygon. */

	CP_E_GradientType					fGradientType;			/*! Type of gradient. */
	SInt16								fGradientCount;			/*! Number of colors in gradient. */
	CP_ARGBColor						fColors[ kCP_MaxGradientColors ];		/*! List of colors in gradient. */
	SInt16								fOffsets[ kCP_MaxGradientColors ];		/*! Offsets used to draw gradient. */
	SInt16								fGradientAngle;			/*! Angle used to draw gradient. */

	float								fSavedScaleAmtHor;		/*! Saved horizontal scale amount. */
	float								fSavedScaleAmtVer;		/*! Saved vertical scale amount. */
	float								fSavedAngle;			/*! Saved rotation angle. */
	float								fSavedSkewAmtHor;		/*! Saved horizontal skew amount. */
	float								fSavedSkewAmtVer;		/*! Saved vertical skew amount. */
};

CPLAT_End_Namespace_CPLAT

#endif	// TARGET_OS_MAC && CP_UseDepreciated_10_4_APIs

#endif	// _H_CP_Graphics_Imp_QD
