//=========================================================================
//
//	FILE:				CP_MLTESupport.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Mix-in class to provide support for the CP_TextView
//						implmentation class based on MacOS MLTE.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 233 $
//						Latest Checkout:  $Date: 2005-07-16 19:41:39 -0700 (Sat, 16 Jul 2005) $
//
//=========================================================================
#ifndef _H_CP_MLTESupport
#define _H_CP_MLTESupport
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_MAC

	// CPLAT_Framework
#include "CP_M_CommandHandler.h"
#include "CP_Point.h"
#include "CP_Size.h"
#include "CP_View.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	\class		CP_MLTESupport
*	\brief		Mix-in class to provide support for the CP_TextView implmentation class based on MacOS MLTE.
*/
class CP_EXPORT CP_MLTESupport
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_MLTESupport();
	//! Destructor.
	virtual								~CP_MLTESupport() throw();
	//@}
			
	//! @name Initialization
	//@{
	static bool							IsInited()											{ return( fsIsInited ); }

			void						InitMLTE();
			void						CreateMLTEObject(
												OSWindow inWindow,
												const CP_Rect& inBounds,
												TXNFrameOptions inFrameOptions,
												TXNFrameType inFrameType,
												TXNFileType inFileType,
												TXNPermanentTextEncodingType inPermanentEncoding,
												OSControl inOwnerControl );
	//@}

	//! @name Appearance
	//@{
			void						ActivateText( bool inActive );
			void						FocusText( bool inFocus );
	//@}

	//! @name Command Handling
	//@{
			void						HandleCommandStatus(
												CP_CommandStatus& inCommandStatus,
												bool& inOutResult,
												CP_CommandNo& inCommandNo );
										
			void						HandleCutCommand( OSStatus& inOutStatus );
			void						HandleCopyCommand( OSStatus& inOutStatus );
			void						HandleClearCommand( OSStatus& inOutStatus );
			void						HandlePasteCommand( OSStatus& inOutStatus );
			void						HandleSelectAllCommand( OSStatus& inOutStatus );
			void						HandleUndoCommand( OSStatus& inOutStatus );
			void						HandleRedoCommand( OSStatus& inOutStatus );
	//@}

	//! @name Drawing
	//@{
			void						Draw();
	//@}

	//! @name Events
	//@{
			ControlPartCode				HandleClick( const CP_Point& inMouseLocation );
	//@}
			
	//! @name Getters
	//@{
			TXNObject					GetTXNObject()										{ return( fTXNObject ); }
			TXNFrameID					GetTXNFrameID()										{ return( fTXNFrameID ); }
			CP_Rect						GetTextArea()										{ return( fTextArea ); }
			CP_Rect						GetFocusOutline()									{ return( fFocusOutline ); }
			CP_Rect						GetTextOutline()									{ return( fTextOutline ); }

			UInt32						GetTextLength();
			CP_String					GetText( char inLineEnd = CP_LF );

			bool						GetDrawFrameWhenFocused()							{ return( fDrawFrameWhenFocused ); }
	//@}

	//! @name Printing
	//@{
			void						Print();
	//@}

	//! @name Selection
	//@{
			void						GetSelection(
												UInt32 *inStartSelection,
												UInt32 *inEndSelection );
			void						SetSelection(
												UInt32 inStartSelection,
												UInt32 inEndSelection );
			bool						HasSelection();
	//@}

	//! @name Setters
	//@{
			void						SetTXNObject( TXNObject inObject )					{ fTXNObject = inObject; }
			void						SetTXNFrameID( TXNFrameID inFrameID )				{ fTXNFrameID = inFrameID; }

			void						SetText( const CP_String& inText );
			void						SetText( CP_InStream *inStream );
			void						InsertText( const CP_String& inText );
			void						InsertText(
												const char *inTextPtr,
												SInt32 inTextLength );
			void						SetPortBounds( const CP_Rect& inBounds );
			void						SetFontSpec( CP_FontSpec& inFontSpec );

			void						SetActive( bool inActive )							{ fIsActive = inActive; }
			void						SetTEActive( bool inTEActive )						{ fTEActive = inTEActive; }
			void						SetFocused( bool inFocused )						{ fIsFocued = inFocused; }

			void						SetTextArea( const CP_Rect inTextArea )				{ fTextArea = inTextArea; }
			void						SetFocusOutline( const CP_Rect inFocusOutline )		{ fFocusOutline = inFocusOutline; }
			void						SetTextOutline( const CP_Rect inTextOutline )		{ fTextOutline = inTextOutline; }

			void						SetDrawFrameWhenFocused( bool inDrawFrame )			{ fDrawFrameWhenFocused = inDrawFrame; }
	//@}

	//! @name Testers
	//@{
			bool						IsActive()											{ return( fIsActive ); }
			bool						IsTEActive()										{ return( fTEActive ); }
			bool						IsFocused()											{ return( fIsFocued ); }
	//@}

protected:

private:
			void						SetFontName( const CP_String& inFontName );
			void						SetFontSize( SInt16 inFontSize );
			void						SetFontStyle( SInt16 inFontStyle );

	// class data
protected:
	static bool							fsIsInited;
	
	OSControl							fOwnerControl;
	
	TXNObject							fTXNObject; 				/*! the txn record */
	TXNFrameID							fTXNFrameID; 				/*! the txn frame ID */

	CP_Rect								fTextArea;
	CP_Rect								fFocusOutline;				/*! rectangle used to draw the focus box */
	CP_Rect								fTextOutline;				/*! rectangle used to draw the border */

	CP_Region							fTextBackgroundRgn;
	
	bool								fIsActive;
	bool								fTEActive;
	bool								fIsFocued;

	bool								fDrawFrameWhenFocused;

	CGrafPtr							fDrawingEnvironment;		/*! grafport where control is drawn */
};

CPLAT_End_Namespace_CPLAT

#endif	// TARGET_OS_MAC

#endif	// _H_CP_MLTESupport

