//=========================================================================
//
//	FILE:				CP_RadioGroup.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			CP_PlatformControl subclass to manage a group of radio buttons.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_RadioGroup
#define _H_CP_RadioGroup
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_PlatformControl.h"
#include "CP_RadioButton.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_RadioGroup
*	@brief		Implementation of a view to manage a group of radio buttons.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_RadioGroup : public CP_PlatformControl
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_RadioGroup();
										CP_RadioGroup(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize );
	//! Destructor.
	virtual								~CP_RadioGroup() throw();
	//@}
			
	//! @name Setters
	//@{
			UInt32						GetCurrentRadioByID();
	//@}

	//! @name Persistance
	//@{
	static CP_M_Persist					*PersistCreator();
	//@}

	//! @name Setters
	//@{
			void						SetCurrentRadioByID( UInt32 inRadioButtonID );
	//@}

protected:
	//! @name Events
	//@{
	virtual	void						InitEventHandler( CP_EventHandler& inEventHandler );
	//@}

	//! @name Persistance
	//@{
	virtual	void						OnInitPersist(
												CP_View *inSuperview,
												CP_DataValueList& inDataValueList );
	//@}

	// class data
protected:
										
	#if TARGET_OS_LINUX
		GSList							*fGTKRadioGroupList;		/*! Linux list of controls. */
	#endif
		
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize );
	//@}
#endif

#if TARGET_OS_WIN32
public:
			void						RegisterButton( CP_RadioButton *inRadioButton );

	virtual	void						SetValue( SInt32 inValue );

private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize );
	//@}

	//! @name Slots
	//@{
			void						ControlClicked(
												CP_DataValueHolder& inValue,
												OSStatus& inOutStatus );
	//@}

#endif

#if TARGET_OS_LINUX
public:
			GSList						*GetGtkRadioGroupList()								{ return( fGTKRadioGroupList ); }
			void						AddRadioButton( CP_RadioButton *inButton );
			
	virtual	void						SetValue( SInt32 inValue );

private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize );
	//@}
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_RadioGroup

