//=========================================================================
//
//	FILE:				CP_SegmentView.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Implementation of a platforms segment view control.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_SegmentView
#define _H_CP_SegmentView
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_PlatformControl.h"

CPLAT_Begin_Namespace_CPLAT

// ===================================================================================
//	CP_S_SegmentViewItemInfo - structure passed into our window list header routines with
//		information about a column in the list.
// ===================================================================================
struct CP_S_SegmentViewItemInfo
{
	CP_String				fText;
	SInt16					fImageResID;
	
	SInt16					fColumnWidth;
	
	UInt32					fCommandID;
	SInt32					fUserData;
	
							CP_S_SegmentViewItemInfo();
							CP_S_SegmentViewItemInfo(
									const CP_String& inText,
									UInt32 inCommandID = CP_CMD_NoCommand,
									const SInt16 inImageResID = kCP_NoImageIndex,
									const SInt16 inColumnWidth = 10,
									SInt32 inUserData = 0 );
							CP_S_SegmentViewItemInfo( const CP_S_SegmentViewItemInfo& inTabProperty );

	CP_S_SegmentViewItemInfo&	operator =( const CP_S_SegmentViewItemInfo& inTabProperty );
};

#define kCP_SegmentViewHeaderEdgeOffset			3

	// typedefs
#if TARGET_OS_LINUX
	typedef std::vector<CP_S_SegmentViewItemInfo, std::allocator<CP_S_SegmentViewItemInfo> >			CP_SegmentViewItem_Array;
	typedef std::vector<CP_S_SegmentViewItemInfo, std::allocator<CP_S_SegmentViewItemInfo> >::iterator	CP_SegmentViewItem_Iterator;
#else
	typedef std::vector<CP_S_SegmentViewItemInfo >														CP_SegmentViewItem_Array;
	typedef std::vector<CP_S_SegmentViewItemInfo >::iterator											CP_SegmentViewItem_Iterator;
#endif

/*!
*	@class		CP_SegmentView
*	@brief		Implementation of a platforms segment view control.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_SegmentView : public CP_PlatformControl
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_SegmentView();
										CP_SegmentView(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize );
	//! Destructor.
	virtual								~CP_SegmentView() throw();
	//@}
			
	//! @name Items
	//@{
			void						AddItem( CP_S_SegmentViewItemInfo& inItemInfo );
			void						InsertItem(
												CP_S_SegmentViewItemInfo& inItemInfo,
												UInt32 inIndex );
			void						DeleteItem( UInt32 inIndex );

			void						SetItem(
												const CP_S_SegmentViewItemInfo& inItemInfo,
												UInt32 inIndex );
			CP_S_SegmentViewItemInfo	GetItem( const UInt32 inColumn );

			UInt32						GetItemCount();
	//@}

	//! @name Persistance
	//@{
	static CP_M_Persist					*PersistCreator();
	//@}
			
protected:
	//! @name Events
	//@{
	virtual	void						InitEventHandler( CP_EventHandler& inEventHandler );
	//@}

	//! @name Persistance
	//@{
	virtual	void						OnInitPersist(
												CP_View *inSuperview,
												CP_DataValueList& inDataValueList );
	//@}

	//! @name Slots
	//@{
			void						HandleControlHitEvent( CP_Event_Base& inEvent );
			void						HandleMouseUpEvent( CP_Event_Base& inEvent );
	//@}

	// class data
protected:
	CP_SegmentViewItem_Array			*fItems;

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
	//! @name Drawing
	//@{
	virtual	void						OnDraw(
												CP_Canvas& inCanvas,
												CP_Region& inUpdateArea );

			void						DrawOneItem(
												CP_Canvas& inCanvas,
												CP_Rect& inBounds,
												const CP_S_SegmentViewItemInfo& inColumnInfo,
												bool inIsSelected,
												bool inIsPressed );
	//@}

private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize );
	//@}

	//! @name Events
	//@{
			void						HandleClickInColumn( const CP_Point& inLocalMouseClick );
	//@}

	//! @name Misc
	//@{
			SInt16						FindColumnFromPoint( const CP_Point& inLocalMouseClick );
			CP_Rect 					GetItemBounds( SInt32 inItem );
	//@}
#endif

#if TARGET_OS_WIN32
	//! @name Drawing
	//@{
	virtual	void						OnDraw(
												CP_Canvas& inCanvas,
												CP_Region& inUpdateArea );

			void						DrawOneItem(
												CP_Canvas& inCanvas,
												CP_Rect& inBounds,
												const CP_S_SegmentViewItemInfo& inColumnInfo,
												bool inIsSelected,
												bool inIsPressed );
	//@}

	//! @name Slots
	//@{
	virtual	void						HandleCommandProcess( CP_Event_Base& inEvent );
	//@}

private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize );
	//@}

	//! @name Events
	//@{
			void						HandleClickInColumn( const CP_Point& inLocalMouseClick );
	//@}

	//! @name Misc
	//@{
			SInt16						FindColumnFromPoint( const CP_Point& inLocalMouseClick );
			CP_Rect 					GetItemBounds( SInt32 inItem );
	//@}
#endif

#if TARGET_OS_LINUX
	//! @name Drawing
	//@{
	virtual	void						OnDraw(
												CP_Canvas& inCanvas,
												CP_Region& inUpdateArea );

			void						DrawOneItem(
												CP_Canvas& inCanvas,
												CP_Rect& inBounds,
												const CP_S_SegmentViewItemInfo& inColumnInfo,
												bool inIsSelected,
												bool inIsPressed );
	//@}

private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize );
	//@}

	//! @name Events
	//@{
			void						HandleClickInColumn( const CP_Point& inLocalMouseClick );
	//@}

	//! @name Misc
	//@{
			SInt16						FindColumnFromPoint( const CP_Point& inLocalMouseClick );
			CP_Rect 					GetItemBounds( SInt32 inItem );
	//@}
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_SegmentView
